from shrub.v3.evg_build_variant import BuildVariant
from shrub.v3.evg_command import EvgCommandType
from shrub.v3.evg_task import EvgTask, EvgTaskRef

from config_generator.etc.distros import compiler_to_vars, find_large_distro, make_distro_str
from config_generator.etc.function import Function
from config_generator.etc.utils import bash_exec

TAG = 'std-matrix'


# pylint: disable=line-too-long
# fmt: off
MATRIX = [
    ('rhel80',     'clang',    None, [99, 11, 17,   ]), # Clang 7.0
    ('rhel84',     'clang',    None, [99, 11, 17, 23]), # Clang 11.0 (max: C2x)
    ('ubuntu2204', 'clang-12', None, [99, 11, 17, 23]), # Clang 12.0 (max: C2x)
    ('rhel90',     'clang',    None, [99, 11, 17, 23]), # Clang 13.0 (max: C2x)
    ('rhel91',     'clang',    None, [99, 11, 17, 23]), # Clang 14.0 (max: C2x)
    ('rhel92',     'clang',    None, [99, 11, 17, 23]), # Clang 15.0 (max: C2x)
    ('rhel93',     'clang',    None, [99, 11, 17, 23]), # Clang 16.0 (max: C2x)
    ('rhel94',     'clang',    None, [99, 11, 17, 23]), # Clang 17.0 (max: C2x)
    ('rhel95',     'clang',    None, [99, 11, 17, 23]), # Clang 18.0 (max: C23)

    ('rhel7-latest',    'gcc',    None, [99, 11,       ]), # GCC 4.8 (max: C11)
    ('rhel80',          'gcc',    None, [99, 11, 17,   ]), # GCC 8.2 (max: C17)
    ('rhel84',          'gcc',    None, [99, 11, 17,   ]), # GCC 8.4 (max: C17)
    ('debian11-latest', 'gcc-10', None, [99, 11, 17, 23]), # GCC 10.2 (max: C2x)
    ('rhel90',          'gcc',    None, [99, 11, 17, 23]), # GCC 11.2 (max: C2x)
    ('rhel92',          'gcc',    None, [99, 11, 17, 23]), # GCC 11.3 (max: C2x)
    ('rhel94',          'gcc',    None, [99, 11, 17, 23]), # GCC 11.4 (max: C2x)
    ('rhel95',          'gcc',    None, [99, 11, 17, 23]), # GCC 11.5 (max: C2x)
    ('ubuntu2404',      'gcc-13', None, [99, 11, 17, 23]), # GCC 13.3 (max: C2x)

    ('macos-14-arm64', 'clang', None, [99, 11, 17, 23]), # Apple Clang

    ('windows-vsCurrent', 'vs2017x64', None, [99, 11,   ]), # Max: C11
    ('windows-vsCurrent', 'vs2019x64', None, [99, 11, 17]), # Max: C17
    ('windows-vsCurrent', 'vs2022x64', None, [99, 11, 17]), # Max: C17
]
# fmt: on
# pylint: enable=line-too-long


class StdCompile(Function):
    name = 'std-compile'
    commands = [
        bash_exec(
            command_type=EvgCommandType.TEST,
            add_expansions_to_env=True,
            working_dir='mongoc',
            script='.evergreen/scripts/compile-std.sh',
        ),
    ]


def functions():
    return StdCompile.defn()


def tasks():
    res = []

    for distro_name, compiler, arch, stds in MATRIX:
        compiler_type = compiler.split('-')[0]

        tags = [TAG, distro_name, compiler_type, 'compile']

        distro = find_large_distro(distro_name)

        compile_vars = None
        compile_vars = compiler_to_vars(compiler)

        if arch:
            tags.append(arch)
            compile_vars.update({'MARCH': arch})

        distro_str = make_distro_str(distro_name, compiler_type, arch)

        for std in stds:
            with_std = {'C_STD_VERSION': std}

            task_name = f'std-c{std}-{distro_str}-compile'

            res.append(
                EvgTask(
                    name=task_name,
                    run_on=distro.name,
                    tags=tags + [f'std-c{std}'],
                    commands=[StdCompile.call(vars=compile_vars | with_std)],
                )
            )

    return res


def variants():
    return [
        BuildVariant(
            name=TAG,
            display_name=TAG,
            tasks=[EvgTaskRef(name=f'.{TAG}')],
        ),
    ]



from shrub.v3.evg_build_variant import BuildVariant
from shrub.v3.evg_command import EvgCommandType, KeyValueParam, ec2_assume_role, expansions_update
from shrub.v3.evg_task import EvgTask, EvgTaskRef
from shrub.v3.evg_task_group import EvgTaskGroup

from config_generator.components.funcs.fetch_det import FetchDET
from config_generator.components.funcs.fetch_source import FetchSource
from config_generator.components.funcs.run_tests import RunTests
from config_generator.components.sasl.openssl import SaslCyrusOpenSSLCompile
from config_generator.etc.distros import find_small_distro
from config_generator.etc.utils import bash_exec


def task_groups():
    return [
        EvgTaskGroup(
            name='test-oidc-task-group',
            tasks=['oidc-auth-test-task'],
            setup_group_can_fail_task=True,
            teardown_group_can_fail_task=True,
            teardown_group_timeout_secs=180,  # 3 minutes
            setup_group=[
                FetchDET.call(),
                ec2_assume_role(role_arn='${aws_test_secrets_role}'),
                bash_exec(
                    command_type=EvgCommandType.SETUP,
                    include_expansions_in_env=['AWS_ACCESS_KEY_ID', 'AWS_SECRET_ACCESS_KEY', 'AWS_SESSION_TOKEN'],
                    script='./drivers-evergreen-tools/.evergreen/auth_oidc/setup.sh',
                ),
            ],
            teardown_group=[
                bash_exec(
                    command_type=EvgCommandType.SETUP,
                    script='./drivers-evergreen-tools/.evergreen/auth_oidc/teardown.sh',
                )
            ],
        ),
        EvgTaskGroup(
            name='test-oidc-azure-task-group',
            tasks=['oidc-azure-auth-test-task'],
            setup_group_can_fail_task=True,
            teardown_group_can_fail_task=True,
            teardown_group_timeout_secs=180,  # 3 minutes
            setup_group=[
                FetchDET.call(),
                ec2_assume_role(role_arn='${aws_test_secrets_role}'),
                bash_exec(
                    command_type=EvgCommandType.SETUP,
                    include_expansions_in_env=['AWS_ACCESS_KEY_ID', 'AWS_SECRET_ACCESS_KEY', 'AWS_SESSION_TOKEN'],
                    env={'AZUREOIDC_VMNAME_PREFIX': 'CDRIVER'},
                    script='./drivers-evergreen-tools/.evergreen/auth_oidc/azure/create-and-setup-vm.sh',
                ),
            ],
            teardown_group=[
                bash_exec(
                    command_type=EvgCommandType.SETUP,
                    script='./drivers-evergreen-tools/.evergreen/auth_oidc/azure/delete-vm.sh',
                ),
            ],
        ),
        EvgTaskGroup(
            name='test-oidc-gcp-task-group',
            tasks=['oidc-gcp-auth-test-task'],
            setup_group_can_fail_task=True,
            teardown_group_can_fail_task=True,
            teardown_group_timeout_secs=180,  # 3 minutes
            setup_group=[
                FetchDET.call(),
                ec2_assume_role(role_arn='${aws_test_secrets_role}'),
                bash_exec(
                    command_type=EvgCommandType.SETUP,
                    include_expansions_in_env=['AWS_ACCESS_KEY_ID', 'AWS_SECRET_ACCESS_KEY', 'AWS_SESSION_TOKEN'],
                    env={'GCPOIDC_VMNAME_PREFIX': 'CDRIVER'},
                    script='./drivers-evergreen-tools/.evergreen/auth_oidc/gcp/setup.sh',
                ),
            ],
            teardown_group=[
                bash_exec(
                    command_type=EvgCommandType.SETUP,
                    script='./drivers-evergreen-tools/.evergreen/auth_oidc/gcp/teardown.sh',
                ),
            ],
        ),
    ]


def tasks():
    return [
        EvgTask(
            name='oidc-auth-test-task',
            run_on=[find_small_distro('ubuntu2404').name],
            commands=[
                FetchSource.call(),
                SaslCyrusOpenSSLCompile.call(),
                expansions_update(
                    updates=[
                        KeyValueParam(key='CC', value='clang'),
                        # OIDC test servers support both OIDC and user/password.
                        KeyValueParam(key='AUTH', value='auth'),  # Use user/password for default test clients.
                        KeyValueParam(key='OIDC', value='oidc'),  # Enable OIDC tests.
                        KeyValueParam(key='MONGODB_VERSION', value='latest'),
                        KeyValueParam(key='TOPOLOGY', value='replica_set'),
                    ]
                ),
                RunTests.call(),
            ],
        ),
        EvgTask(
            name='oidc-azure-auth-test-task',
            run_on=['debian11-small'],  # TODO: switch to 'debian11-latest' after DEVPROD-23011 fixed.
            commands=[
                FetchSource.call(),
                bash_exec(
                    working_dir='mongoc',
                    add_expansions_to_env=True,
                    command_type=EvgCommandType.TEST,
                    script='.evergreen/scripts/oidc-azure-compile.sh',
                ),
                expansions_update(file='mongoc/oidc-remote-test-expansion.yml'),
                bash_exec(
                    add_expansions_to_env=True,
                    command_type=EvgCommandType.TEST,
                    env={
                        'AZUREOIDC_DRIVERS_TAR_FILE': '${OIDC_TEST_TARBALL}',
                        'AZUREOIDC_TEST_CMD': 'source ./env.sh && ./.evergreen/scripts/oidc-azure-test.sh',
                    },
                    script='./drivers-evergreen-tools/.evergreen/auth_oidc/azure/run-driver-test.sh',
                ),
            ],
        ),
        EvgTask(
            name='oidc-gcp-auth-test-task',
            run_on=['debian11-small'],  # TODO: switch to 'debian11-latest' after DEVPROD-23011 fixed.
            commands=[
                FetchSource.call(),
                bash_exec(
                    working_dir='mongoc',
                    add_expansions_to_env=True,
                    command_type=EvgCommandType.TEST,
                    script='.evergreen/scripts/oidc-gcp-compile.sh',
                ),
                expansions_update(file='mongoc/oidc-remote-test-expansion.yml'),
                bash_exec(
                    add_expansions_to_env=True,
                    command_type=EvgCommandType.TEST,
                    env={
                        'GCPOIDC_DRIVERS_TAR_FILE': '${OIDC_TEST_TARBALL}',
                        'GCPOIDC_TEST_CMD': 'source ./secrets-export.sh && ./.evergreen/scripts/oidc-gcp-test.sh',
                    },
                    script='./drivers-evergreen-tools/.evergreen/auth_oidc/gcp/run-driver-test.sh',
                ),
            ],
        ),
    ]


def variants():
    return [
        BuildVariant(
            name='oidc',
            display_name='OIDC',
            tasks=[
                EvgTaskRef(name='test-oidc-task-group'),
                EvgTaskRef(name='test-oidc-azure-task-group'),
                EvgTaskRef(name='test-oidc-gcp-task-group'),
            ],
        ),
    ]



from itertools import product

from shrub.v3.evg_build_variant import BuildVariant
from shrub.v3.evg_command import EvgCommandType, FunctionCall
from shrub.v3.evg_task import EvgTask, EvgTaskRef

from config_generator.components.funcs.fetch_source import FetchSource
from config_generator.etc.distros import find_large_distro, make_distro_str
from config_generator.etc.function import Function
from config_generator.etc.utils import bash_exec

TAG = 'openssl-compat'

# pylint: disable=line-too-long
# fmt: off
OPENSSL_MATRIX = [
    ('ubuntu2404', 'gcc', ['shared', 'static'], ['1.0.2', '1.1.1', '3.0.9', '3.1.2', '3.2.5', '3.3.4', '3.4.2', '3.5.1']),
]
# fmt: on

# pylint: disable=line-too-long
# fmt: off
OPENSSL_FIPS_MATRIX = [
    # https://openssl-library.org/source/
    # > The following OpenSSL version(s) are FIPS validated:
    # > - 3.1.2: FIPS 140-3
    # > - 3.0.9: FIPS 140-2
    # > - ...
    ('ubuntu2404', 'gcc', ['shared', 'static'], ['3.0.9', '3.1.2']),
]
# fmt: on


class OpenSSLSetup(Function):
    name = 'openssl-compat'
    commands = [
        bash_exec(
            command_type=EvgCommandType.SETUP,
            working_dir='mongoc',
            include_expansions_in_env=[
                'OPENSSL_ENABLE_FIPS',
                'OPENSSL_USE_STATIC_LIBS',
                'OPENSSL_VERSION',
            ],
            script='.evergreen/scripts/openssl-compat-setup.sh',
        ),
        bash_exec(
            command_type=EvgCommandType.SETUP,
            working_dir='mongoc',
            include_expansions_in_env=[
                'OPENSSL_USE_STATIC_LIBS',
                'OPENSSL_VERSION',
            ],
            script='.evergreen/scripts/openssl-compat-check.sh',
        ),
    ]


def functions():
    return OpenSSLSetup.defn()


def tasks():
    for distro_name, compiler, link_types, versions in OPENSSL_MATRIX:
        distro_str = make_distro_str(distro_name, compiler, None)

        for link_type, version in product(link_types, versions):
            vars = {'OPENSSL_VERSION': version}

            if link_type == 'static':
                vars |= {'OPENSSL_USE_STATIC_LIBS': 'ON'}

            yield EvgTask(
                name=f'{TAG}-{version}-{link_type}-{distro_str}',
                run_on=find_large_distro(distro_name).name,
                tags=[TAG, f'openssl-{version}', f'openssl-{link_type}', distro_name, compiler],
                commands=[
                    FetchSource.call(),
                    OpenSSLSetup.call(vars=vars),
                    FunctionCall(func='run auth tests'),
                ],
            )

    for distro_name, compiler, link_types, versions in OPENSSL_FIPS_MATRIX:
        distro_str = make_distro_str(distro_name, compiler, None)

        for link_type, version in product(link_types, versions):
            vars = {'OPENSSL_VERSION': version, 'OPENSSL_ENABLE_FIPS': 'ON'}

            if link_type == 'static':
                vars |= {'OPENSSL_USE_STATIC_LIBS': 'ON'}

            yield EvgTask(
                name=f'{TAG}-fips-{version}-{link_type}-{distro_str}',
                run_on=find_large_distro(distro_name).name,
                tags=[TAG, f'openssl-fips-{version}', f'openssl-{link_type}', distro_name, compiler],
                commands=[
                    FetchSource.call(),
                    OpenSSLSetup.call(vars=vars),
                    FunctionCall(func='run auth tests'),
                ],
            )


def variants():
    return [
        BuildVariant(
            name=f'{TAG}-matrix',
            display_name='OpenSSL Compatibility Matrix',
            tasks=[EvgTaskRef(name=f'.{TAG}')],
        ),
    ]



from shrub.v3.evg_command import EvgCommandType

from config_generator.etc.function import Function
from config_generator.etc.utils import bash_exec


class FetchDET(Function):
    name = 'fetch-det'
    commands = [
        bash_exec(
            command_type=EvgCommandType.SETUP,
            script="""\
                if [[ ! -d drivers-evergreen-tools ]]; then
                    git clone --depth=1 https://github.com/mongodb-labs/drivers-evergreen-tools.git
                fi
            """,
        ),
        # Make shell scripts executable.
        bash_exec(
            command_type=EvgCommandType.SETUP,
            working_dir='drivers-evergreen-tools',
            script='find .evergreen -type f -name "*.sh" -exec chmod +rx "{}" \;',
        ),
    ]


def functions():
    return FetchDET.defn()



from shrub.v3.evg_build_variant import BuildVariant
from shrub.v3.evg_command import EvgCommandType
from shrub.v3.evg_task import EvgTask, EvgTaskRef

from config_generator.etc.distros import find_small_distro
from config_generator.etc.function import Function
from config_generator.etc.utils import bash_exec

TAG = 'clang-format'


class ClangFormat(Function):
    name = TAG
    commands = [
        bash_exec(
            command_type=EvgCommandType.TEST,
            working_dir='mongoc',
            env={
                'DRYRUN': '1',
            },
            script='uv run --frozen --only-group=format tools/format.py --mode=check',
        ),
    ]


def functions():
    return ClangFormat.defn()


def tasks():
    yield EvgTask(
        name=TAG,
        tags=[TAG],
        commands=[
            ClangFormat.call(),
        ],
    )


def variants():
    return [
        BuildVariant(
            name=TAG,
            display_name=TAG,
            run_on=[find_small_distro('ubuntu2204').name],
            tasks=[EvgTaskRef(name=f'.{TAG}')],
        ),
    ]



#!/usr/bin/env python3
#
# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Check that public libbson/libmongoc headers all include the prelude line."""

import sys
from pathlib import Path

if len(sys.argv) != 2:
    print('Usage: python check-preludes.py <mongo-c-driver directory>')
    sys.exit(1)

MONGOC_PREFIX = Path('src/libmongoc/src/mongoc')
BSON_PREFIX = Path('src/libbson/src/bson')
COMMON_PREFIX = Path('src/common/src')

checks = [
    {
        'name': 'libmongoc',
        'headers': list(MONGOC_PREFIX.glob('mongoc-*.h')),
        'exclusions': [
            MONGOC_PREFIX / 'mongoc-prelude.h',
            MONGOC_PREFIX / 'mongoc.h',
        ],
        'include': '#include <mongoc/mongoc-prelude.h>',
    },
    {
        'name': 'libbson',
        'headers': list(BSON_PREFIX.glob('bson-*.h')),
        'exclusions': [
            BSON_PREFIX / 'bson-prelude.h',
            BSON_PREFIX / 'bson.h',
        ],
        'include': '#include <bson/bson-prelude.h>',
    },
    {
        'name': 'common',
        'headers': list(COMMON_PREFIX.glob('*.h')),
        'exclusions': [COMMON_PREFIX / 'common-prelude.h'],
        'include': '#include <common-prelude.h>',
    },
]

for check in checks:
    NAME = check['name']
    print(f'Checking headers for {NAME}')
    assert len(check['headers']) > 0
    for header in check['headers']:
        if header in check['exclusions'] or header.name.endswith('-private.h'):
            continue
        lines = Path(header).read_text(encoding='utf-8').splitlines()
        if check['include'] not in lines:
            print(f'{header} did not include prelude')
            sys.exit(1)

print('All checks passed')



# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import OrderedDict as OD

from evergreen_config_generator.functions import Function, s3_put

from evergreen_config_lib import shell_mongoc

build_path = '${build_variant}/${revision}/${version_id}/${build_id}'

all_functions = OD(
    [
        (
            'install ssl',
            Function(
                shell_mongoc(
                    '.evergreen/scripts/install-ssl.sh',
                    test=False,
                    add_expansions_to_env=True,
                ),
            ),
        ),
        (
            'upload coverage',
            Function(
                shell_mongoc(
                    r"""
                    export AWS_ACCESS_KEY_ID=${aws_key}
                    export AWS_SECRET_ACCESS_KEY=${aws_secret}
                    aws s3 cp coverage s3://mciuploads/${project}/%s/coverage/ --recursive --acl public-read --region us-east-1
                    """
                    % (build_path,),
                    test=False,
                    silent=True,
                ),
                s3_put(
                    build_path + '/coverage/index.html',
                    aws_key='${aws_key}',
                    aws_secret='${aws_secret}',
                    local_file='mongoc/coverage/index.html',
                    bucket='mciuploads',
                    permissions='public-read',
                    content_type='text/html',
                    display_name='Coverage Report',
                ),
            ),
        ),
        (
            'upload scan artifacts',
            Function(
                shell_mongoc(
                    r"""
                    if find scan -name \*.html | grep -q html; then
                      (cd scan && find . -name index.html -exec echo "<li><a href='{}'>{}</a></li>" \;) >> scan.html
                    else
                      echo "No issues found" > scan.html
                    fi
                    """,
                ),
                shell_mongoc(
                    r"""
                    export AWS_ACCESS_KEY_ID=${aws_key}
                    export AWS_SECRET_ACCESS_KEY=${aws_secret}
                    aws s3 cp scan s3://mciuploads/${project}/%s/scan/ --recursive --acl public-read --region us-east-1
                    """
                    % (build_path,),
                    test=False,
                    silent=True,
                ),
                s3_put(
                    build_path + '/scan/index.html',
                    aws_key='${aws_key}',
                    aws_secret='${aws_secret}',
                    local_file='mongoc/scan.html',
                    bucket='mciuploads',
                    permissions='public-read',
                    content_type='text/html',
                    display_name='Scan Build Report',
                ),
            ),
        ),
        # Use "silent=True" to hide output since errors may contain credentials.
        (
            'run auth tests',
            Function(
                shell_mongoc(
                    '.evergreen/scripts/run-auth-tests.sh',
                    add_expansions_to_env=True,
                ),
            ),
        ),
        (
            'link sample program',
            Function(
                shell_mongoc(
                    r"""
                    # Compile a program that links dynamically or statically to libmongoc,
                    # using variables from pkg-config or CMake's find_package command.
                    export BUILD_SAMPLE_WITH_CMAKE=${BUILD_SAMPLE_WITH_CMAKE}
                    export ENABLE_SSL=${ENABLE_SSL}
                    export ENABLE_SNAPPY=${ENABLE_SNAPPY}
                    LINK_STATIC=  .evergreen/scripts/link-sample-program.sh
                    LINK_STATIC=1 .evergreen/scripts/link-sample-program.sh
                    """,
                    include_expansions_in_env=['distro_id'],
                ),
            ),
        ),
        (
            'link sample program bson',
            Function(
                shell_mongoc(
                    r"""
                    # Compile a program that links dynamically or statically to libbson,
                    # using variables from pkg-config or from CMake's find_package command.
                    BUILD_SAMPLE_WITH_CMAKE=  LINK_STATIC=  .evergreen/scripts/link-sample-program-bson.sh
                    BUILD_SAMPLE_WITH_CMAKE=  LINK_STATIC=1 .evergreen/scripts/link-sample-program-bson.sh
                    BUILD_SAMPLE_WITH_CMAKE=1 LINK_STATIC=  .evergreen/scripts/link-sample-program-bson.sh
                    BUILD_SAMPLE_WITH_CMAKE=1 LINK_STATIC=1 .evergreen/scripts/link-sample-program-bson.sh
                    """,
                    include_expansions_in_env=['distro_id'],
                ),
            ),
        ),
        (
            'link sample program MSVC',
            Function(
                shell_mongoc(
                    r"""
                    # Build libmongoc with CMake and compile a program that links
                    # dynamically or statically to it, using variables from CMake's
                    # find_package command.
                    export ENABLE_SSL=${ENABLE_SSL}
                    export ENABLE_SNAPPY=${ENABLE_SNAPPY}
                    LINK_STATIC=  cmd.exe /c .\\.evergreen\\scripts\\link-sample-program-msvc.cmd
                    LINK_STATIC=1 cmd.exe /c .\\.evergreen\\scripts\\link-sample-program-msvc.cmd
                    """,
                )
            ),
        ),
        (
            'link sample program mingw',
            Function(
                shell_mongoc(
                    r"""
                    # Build libmongoc with CMake and compile a program that links
                    # dynamically to it, using variables from pkg-config.exe.
                    PATH="/cygdrive/c/ProgramData/chocolatey/lib/winlibs/tools/mingw64/bin:$PATH" # mingw-w64 GCC
                    cmd.exe /c .\\.evergreen\\scripts\\link-sample-program-mingw.cmd
                    """,
                )
            ),
        ),
        (
            'link sample program MSVC bson',
            Function(
                shell_mongoc(
                    r"""
                    # Build libmongoc with CMake and compile a program that links
                    # dynamically or statically to it, using variables from CMake's
                    # find_package command.
                    export ENABLE_SSL=${ENABLE_SSL}
                    export ENABLE_SNAPPY=${ENABLE_SNAPPY}
                    LINK_STATIC=  cmd.exe /c .\\.evergreen\\scripts\\link-sample-program-msvc-bson.cmd
                    LINK_STATIC=1 cmd.exe /c .\\.evergreen\\scripts\\link-sample-program-msvc-bson.cmd
                    """,
                )
            ),
        ),
        (
            'link sample program mingw bson',
            Function(
                shell_mongoc(
                    r"""
                    # Build libmongoc with CMake and compile a program that links
                    # dynamically to it, using variables from pkg-config.exe.
                    PATH="/cygdrive/c/ProgramData/chocolatey/lib/winlibs/tools/mingw64/bin:$PATH" # mingw-w64 GCC
                    cmd.exe /c .\\.evergreen\\scripts\\link-sample-program-mingw-bson.cmd
                    """,
                )
            ),
        ),
        (
            'update codecov.io',
            Function(
                shell_mongoc(
                    r"""
                    # Note: coverage is currently only enabled on the ubuntu1804 distro.
                    # This script does not support MacOS, Windows, or non-x86_64 distros.
                    # Update accordingly if code coverage is expanded to other distros.
                    curl -Os https://uploader.codecov.io/latest/linux/codecov
                    chmod +x codecov
                    # -Z: Exit with a non-zero value if error.
                    # -g: Run with gcov support.
                    # -t: Codecov upload token.
                    # perl: filter verbose "Found" list and "Processing" messages.
                    ./codecov -Zgt "${codecov_token}" | perl -lne 'print if not m|^.*\.gcov(\.\.\.)?$|'
                    """,
                    test=False,
                ),
            ),
        ),
        (
            'compile coverage',
            Function(
                shell_mongoc(
                    'COVERAGE=ON .evergreen/scripts/compile.sh',
                    add_expansions_to_env=True,
                ),
            ),
        ),
        (
            'run aws tests',
            Function(
                # Assume role to get AWS secrets.
                {'command': 'ec2.assume_role', 'params': {'role_arn': '${aws_test_secrets_role}'}},
                shell_mongoc(
                    r"""
                    pushd ../drivers-evergreen-tools/.evergreen/auth_aws
                    ./setup_secrets.sh drivers/aws_auth
                    popd # ../drivers-evergreen-tools/.evergreen/auth_aws
                    """,
                    include_expansions_in_env=['AWS_ACCESS_KEY_ID', 'AWS_SECRET_ACCESS_KEY', 'AWS_SESSION_TOKEN'],
                ),
                shell_mongoc(
                    r"""
                    pushd ../drivers-evergreen-tools/.evergreen/auth_aws
                    . ./activate-authawsvenv.sh
                    popd # ../drivers-evergreen-tools/.evergreen/auth_aws
                    .evergreen/scripts/run-aws-tests.sh
                    """,
                    include_expansions_in_env=['TESTCASE'],
                ),
            ),
        ),
    ]
)



#!/usr/bin/env python3

# Generates *.yml files under .evergreen/generated_configs.


import sys
from importlib import import_module

GENERATOR_NAMES = [
    'functions',
    'tasks',
    'task_groups',
    'variants',
    'legacy_config',
]


def main():
    # Requires Python 3.10 or newer.
    assert sys.version_info.major >= 3
    assert sys.version_info.minor >= 10

    for name in GENERATOR_NAMES:
        m = import_module(f'config_generator.generators.{name}')
        print(f'Running {name}.generate()...')
        m.generate()
        print(f'Running {name}.generate()... done.')


if __name__ == '__main__':
    main()



# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import OrderedDict as OD
from textwrap import dedent
from typing import Iterable

from evergreen_config_generator import ConfigObject

from . import MutableValueMapping, Value, ValueMapping, ValueOrderedDict


def func(func_name: str, **kwargs: Value) -> MutableValueMapping:
    od: MutableValueMapping = OD([('func', func_name)])
    if kwargs:
        od['vars'] = OD(sorted(kwargs.items()))

    return od


def s3_put(remote_file: str, project_path: bool = True, **kwargs: Value) -> ValueMapping:
    if project_path:
        remote_file = '${project}/' + remote_file

    return ValueOrderedDict(
        [
            ('command', 's3.put'),
            (
                'params',
                ValueOrderedDict(
                    (
                        ('aws_key', '${aws_key}'),
                        ('aws_secret', '${aws_secret}'),
                        ('remote_file', remote_file),
                        ('bucket', 'mciuploads'),
                        ('permissions', 'public-read'),
                        *kwargs.items(),
                    )
                ),
            ),
        ]
    )


def strip_lines(s: str) -> str:
    return '\n'.join(line for line in s.split('\n') if line.strip())


def shell_exec(
    script: str,
    test: bool = True,
    errexit: bool = True,
    xtrace: bool = False,
    silent: bool = False,
    continue_on_err: bool = False,
    working_dir: str | None = None,
    background: bool = False,
    add_expansions_to_env: bool = False,
    redirect_standard_error_to_output: bool = False,
    include_expansions_in_env: Iterable[str] = (),
) -> ValueMapping:
    dedented = ''
    if errexit:
        dedented += 'set -o errexit\n'

    if xtrace:
        dedented += 'set -o xtrace\n'

    dedented += dedent(strip_lines(script))
    command = ValueOrderedDict([('command', 'shell.exec')])
    if test:
        command['type'] = 'test'

    command['params'] = OD()
    if silent:
        command['params']['silent'] = True

    if working_dir is not None:
        command['params']['working_dir'] = working_dir

    if continue_on_err:
        command['params']['continue_on_err'] = True

    if background:
        command['params']['background'] = True

    if add_expansions_to_env:
        command['params']['add_expansions_to_env'] = True

    if redirect_standard_error_to_output:
        command['params']['redirect_standard_error_to_output'] = True

    if include_expansions_in_env:
        command['params']['include_expansions_in_env'] = list(include_expansions_in_env)

    command['params']['shell'] = 'bash'
    command['params']['script'] = dedented
    return command


def targz_pack(target: str, source_dir: str, *include: str) -> ValueMapping:
    return OD(
        [
            ('command', 'archive.targz_pack'),
            ('params', OD([('target', target), ('source_dir', source_dir), ('include', list(include))])),
        ]
    )


class Function(ConfigObject):
    def __init__(self, *commands: Value):
        super(Function, self).__init__()
        self.commands = commands

    def to_dict(self) -> Value:
        return list(self.commands)



from shrub.v3.evg_build_variant import BuildVariant

from config_generator.etc.compile import generate_compile_tasks
from config_generator.etc.cse.compile import CompileCommon
from config_generator.etc.cse.test import generate_test_tasks
from config_generator.etc.function import merge_defns
from config_generator.etc.utils import TaskRef

SSL = 'openssl'
TAG = f'cse-matrix-{SSL}'


# pylint: disable=line-too-long
# fmt: off
COMPILE_MATRIX = [
    # For test matrix.
    ('rhel8-latest',       'gcc',       None, ['cyrus']),
    ('rhel8-arm64-latest', 'gcc',       None, ['cyrus']),
    ('rhel8-zseries',      'gcc',       None, ['cyrus']), # Big Endian.
    ('windows-vsCurrent',  'vs2022x64', None, ['cyrus']),

    # For compile only.
    ('debian11-latest', 'clang',    None, ['cyrus']),
    ('debian11-latest', 'gcc',      None, ['cyrus']),
    ('debian12-latest', 'clang',    None, ['cyrus']),
    ('debian12-latest', 'gcc',      None, ['cyrus']),
    ('rhel80',          'gcc',      None, ['cyrus']),
    ('ubuntu2204',      'gcc',      None, ['cyrus']),
    ('ubuntu2204',      'clang-12', None, ['cyrus']),
    ('ubuntu2404',      'gcc',      None, ['cyrus']),
    ('ubuntu2404',      'clang-14', None, ['cyrus']),
]

# QE (subset of CSFLE) requires 7.0+ and are skipped by "server" tasks.
TEST_MATRIX = [
    ('rhel8-latest', 'gcc', None, 'cyrus', ['auth'], ['server', 'replica', 'sharded'], ['4.2', '4.4', '5.0', '6.0', '7.0', '8.0', 'latest']),

    # rhel8-arm64 only provides 4.4+.
    ('rhel8-arm64-latest', 'gcc', None, 'cyrus', ['auth'], ['server', 'replica', 'sharded'], ['4.4', '5.0', '6.0', '7.0', '8.0', 'latest']),

    # rhel8-zseries only provides 5.0+. Resource-limited: use sparingly.
    ('rhel8-zseries', 'gcc', None, 'cyrus', ['auth'], ['sharded'], ['5.0', 'latest']),

    ('windows-vsCurrent', 'vs2022x64', None, 'cyrus', ['auth'], ['server', 'replica', 'sharded'], ['4.2', '4.4', '5.0', '6.0', '7.0', '8.0', 'latest']),
]
# fmt: on
# pylint: enable=line-too-long


class OpenSSLCompileCommon(CompileCommon):
    ssl = 'OPENSSL'


class SaslCyrusOpenSSLCompile(OpenSSLCompileCommon):
    name = 'cse-sasl-cyrus-openssl-compile'
    commands = OpenSSLCompileCommon.compile_commands(sasl='CYRUS')


def functions():
    return merge_defns(
        SaslCyrusOpenSSLCompile.defn(),
    )


SASL_TO_FUNC = {
    'cyrus': SaslCyrusOpenSSLCompile,
}

MORE_TAGS = ['cse']

TASKS = [
    *generate_compile_tasks(SSL, TAG, SASL_TO_FUNC, COMPILE_MATRIX, MORE_TAGS),
    *generate_test_tasks(SSL, TAG, TEST_MATRIX),
]


def tasks():
    res = TASKS.copy()

    # PowerPC and zSeries are limited resources.
    for task in res:
        if any(pattern in task.run_on for pattern in ['power', 'zseries']):
            task.patchable = False

    return res


def variants():
    expansions = {
        'CLIENT_SIDE_ENCRYPTION': 'on',
    }

    tasks = []

    # PowerPC and zSeries are limited resources.
    for task in TASKS:
        if any(pattern in task.run_on for pattern in ['power', 'zseries']):
            tasks.append(
                TaskRef(
                    name=task.name,
                    batchtime=1440,  # 1 day
                )
            )
        else:
            tasks.append(task.get_task_ref())

    tasks.sort(key=lambda t: t.name)

    return [
        BuildVariant(
            name=TAG,
            display_name=TAG,
            tasks=tasks,
            expansions=expansions,
        ),
    ]



from typing import Literal

from packaging.version import Version
from pydantic import BaseModel, validator


class Distro(BaseModel):
    """
    Defines common properties of a given Evergreen distro.

    * name: Name of the distro.
    * os: Name of the operating system.
    * os_type: One of Linux, MacOS, or Windows.
    * os_ver: Version of the operating system.
    * vs_ver: Version of Visual Studio available.
    * size: Size of tasks the distro is designed to handle.
    * arch: Target architecture.
    """

    name: str
    os: str | None = None
    os_type: Literal['linux', 'macos', 'windows'] | None = None
    os_ver: str | None = None
    vs_ver: (
        Literal[
            '2017',
            '2019',
            '2022',
            'vsCurrent',
        ]
        | None
    ) = None
    size: Literal['small', 'large'] | None = None
    arch: Literal['arm64', 'power', 'zseries'] | None = None

    @validator('os_ver')
    @classmethod
    def validate_os_ver(cls, value):
        return value == 'latest' or Version(value)


def ls_distro(name, **kwargs):
    return [
        Distro(name=f'{name}-large', size='large', **kwargs),
        Distro(name=f'{name}-small', size='small', **kwargs),
    ]


DEBIAN_DISTROS = [
    *ls_distro(name='debian11-latest', os='debian', os_type='linux', os_ver='11'),
    *ls_distro(name='debian12-latest', os='debian', os_type='linux', os_ver='12'),
]

MACOS_DISTROS = [
    Distro(name='macos-14', os='macos', os_type='macos', os_ver='14'),
]

MACOS_ARM64_DISTROS = [
    Distro(name='macos-14-arm64', os='macos', os_type='macos', os_ver='14', arch='arm64'),
]

RHEL_DISTROS = [
    *ls_distro(name='rhel7-latest', os='rhel', os_type='linux', os_ver='7'),
    *ls_distro(name='rhel8-latest', os='rhel', os_type='linux', os_ver='8'),
    *ls_distro(name='rhel80', os='rhel', os_type='linux', os_ver='8.0'),
    *ls_distro(name='rhel84', os='rhel', os_type='linux', os_ver='8.4'),
    *ls_distro(name='rhel90', os='rhel', os_type='linux', os_ver='9.0'),
    *ls_distro(name='rhel91', os='rhel', os_type='linux', os_ver='9.1'),
    *ls_distro(name='rhel92', os='rhel', os_type='linux', os_ver='9.2'),
    *ls_distro(name='rhel93', os='rhel', os_type='linux', os_ver='9.3'),
    *ls_distro(name='rhel94', os='rhel', os_type='linux', os_ver='9.4'),
    *ls_distro(name='rhel95', os='rhel', os_type='linux', os_ver='9.5'),  # rhel9-latest
]

RHEL_ARM64_DISTROS = [
    *ls_distro(name='rhel8-arm64-latest', os='rhel', os_type='linux', os_ver='8', arch='arm64'),
]

RHEL_POWER_DISTROS = [
    *ls_distro(name='rhel8-power', os='rhel', os_type='linux', os_ver='8', arch='power'),
]

RHEL_ZSERIES_DISTROS = [
    *ls_distro(name='rhel8-zseries', os='rhel', os_type='linux', os_ver='8', arch='zseries'),
]

UBUNTU_DISTROS = [
    *ls_distro(name='ubuntu2004', os='ubuntu', os_type='linux', os_ver='20.04'),
    *ls_distro(name='ubuntu2204', os='ubuntu', os_type='linux', os_ver='22.04'),
    *ls_distro(name='ubuntu2404', os='ubuntu', os_type='linux', os_ver='24.04'),
]

UBUNTU_ARM64_DISTROS = [
    *ls_distro(name='ubuntu2204-arm64', os='ubuntu', os_type='linux', os_ver='22.04', arch='arm64'),
]

WINDOWS_DISTROS = [
    *ls_distro(name='windows-vsCurrent', os='windows', os_type='windows', vs_ver='vsCurrent'),  # Windows Server 2019
]

GRAVITON_DISTROS = [
    Distro(name='amazon2023-arm64-latest-large-m8g', os='amazon2023', os_type='linux', os_ver='2023', arch='arm64'),
    Distro(name='amazon2-arm64-latest-large-m8g', os='amazon2', os_type='linux', os_ver='2', arch='arm64'),
]

# See: https://evergreen.mongodb.com/distros
# Ensure no-arch distros are ordered before arch-specific distros.
ALL_DISTROS = [
    *DEBIAN_DISTROS,
    *GRAVITON_DISTROS,
    *MACOS_DISTROS,
    *MACOS_ARM64_DISTROS,
    *RHEL_DISTROS,
    *RHEL_ARM64_DISTROS,
    *RHEL_POWER_DISTROS,
    *RHEL_ZSERIES_DISTROS,
    *UBUNTU_DISTROS,
    *UBUNTU_ARM64_DISTROS,
    *WINDOWS_DISTROS,
]


def find_distro(name) -> Distro:
    candidates = [d for d in ALL_DISTROS if name == d.name]

    if not candidates:
        raise ValueError(f'could not find a distro with the name {name}')

    return candidates[0]


def find_large_distro(name) -> Distro:
    candidates = [d for d in ALL_DISTROS if f'{name}-large' == d.name]

    if candidates:
        return candidates[0]

    return find_distro(name)


def find_small_distro(name) -> Distro:
    candidates = [d for d in ALL_DISTROS if f'{name}-small' == d.name]

    if candidates:
        return candidates[0]

    return find_distro(name)


def make_distro_str(distro_name, compiler, arch) -> str:
    if distro_name.startswith('windows-vsCurrent'):
        # Rename `windows-vsCurrent-*` distros to `windows-<year>` where`<year>`
        # is the Windows Server version used by the distro, e.g.:
        #     ('windows-vsCurrent-2022', 'vs2017x64', None) -> windows-2022-vs2017-x64
        #     ('windows-vsCurrent-2022', 'mingw',     None) -> windows-2022-mingw
        #     ('windows-vsCurrent',      'vs2017x64', None) -> windows-2019-vs2017-x64
        #     ('windows-vsCurrent',      'mingw',     None) -> windows-2019-mingw
        maybe_arch = compiler[len('vs20XY') :]
        if maybe_arch in ('x86', 'x64'):
            compiler_str = compiler[: -len(maybe_arch)] + '-' + maybe_arch
        else:
            compiler_str = compiler
        if distro_name.startswith('windows-vsCurrent-'):
            distro_str = 'windows-' + distro_name[len('windows-vsCurrent-') :] + f'-{compiler_str}'
        else:
            distro_str = 'windows-2019-' + compiler_str
    else:
        distro_str = distro_name
        if compiler:
            distro_str += f'-{compiler}'

    if arch:
        distro_str += f'-{arch}'

    return distro_str


def to_cc(compiler):
    return {
        'vs2017x64': 'Visual Studio 15 2017',
        'vs2017x86': 'Visual Studio 15 2017',
        'vs2019x64': 'Visual Studio 16 2019',
        'vs2019x86': 'Visual Studio 16 2019',
        'vs2022x64': 'Visual Studio 17 2022',
        'vs2022x86': 'Visual Studio 17 2022',
    }.get(compiler, compiler)


def to_platform(compiler):
    return {
        'vs2017x64': 'x64',
        'vs2017x86': 'Win32',
        'vs2019x64': 'x64',
        'vs2019x86': 'Win32',
        'vs2022x64': 'x64',
        'vs2022x86': 'Win32',
    }.get(compiler, compiler)


def compiler_to_vars(compiler):
    if compiler is None:
        return {}

    match compiler, compiler.split('-'):
        case _, ['gcc', *rest]:
            return {
                'CC': '-'.join(['gcc'] + rest),
                'CXX': '-'.join(['g++'] + rest),
            }

        case _, ['clang', *rest]:
            return {
                'CC': '-'.join(['clang'] + rest),
                'CXX': '-'.join(['clang++'] + rest),
            }

        case str(vs), _ if 'vs' in vs:
            return {
                'CMAKE_GENERATOR': to_cc(vs),
                'CMAKE_GENERATOR_PLATFORM': to_platform(vs),
            }

        case _, ['mingw', *rest]:
            return {
                'CC': '-'.join(['gcc'] + rest),
                'CXX': '-'.join(['g++'] + rest),
            }

        case compiler, _:
            return {
                'CC': compiler,
                'CXX': compiler,
            }



import sys

# `MD_FLAGS` maps the flag to its bit position.
# The bit positions must match those defined in src/mongoc/mongoc-handshake-private.h
MD_FLAGS = {
    'MONGOC_MD_FLAG_ENABLE_CRYPTO': 0,
    'MONGOC_MD_FLAG_ENABLE_CRYPTO_CNG': 1,
    'MONGOC_MD_FLAG_ENABLE_CRYPTO_COMMON_CRYPTO': 2,
    'MONGOC_MD_FLAG_ENABLE_CRYPTO_LIBCRYPTO': 3,
    'MONGOC_MD_FLAG_ENABLE_CRYPTO_SYSTEM_PROFILE': 4,
    'MONGOC_MD_FLAG_ENABLE_SASL': 5,
    'MONGOC_MD_FLAG_ENABLE_SSL': 6,
    'MONGOC_MD_FLAG_ENABLE_SSL_OPENSSL': 7,
    'MONGOC_MD_FLAG_ENABLE_SSL_SECURE_CHANNEL': 8,
    'MONGOC_MD_FLAG_ENABLE_SSL_SECURE_TRANSPORT': 9,
    'MONGOC_MD_FLAG_EXPERIMENTAL_FEATURES': 10,
    'MONGOC_MD_FLAG_HAVE_SASL_CLIENT_DONE': 11,
    'MONGOC_MD_FLAG_HAVE_WEAK_SYMBOLS': 12,
    'MONGOC_MD_FLAG_NO_AUTOMATIC_GLOBALS': 13,  # Removed in CDRIVER-1330.
    'MONGOC_MD_FLAG_ENABLE_SSL_LIBRESSL': 14,  # Removed in CDRIVER-5693.
    'MONGOC_MD_FLAG_ENABLE_SASL_CYRUS': 15,
    'MONGOC_MD_FLAG_ENABLE_SASL_SSPI': 16,
    'MONGOC_MD_FLAG_HAVE_SOCKLEN': 17,
    'MONGOC_MD_FLAG_ENABLE_COMPRESSION': 18,
    'MONGOC_MD_FLAG_ENABLE_COMPRESSION_SNAPPY': 19,
    'MONGOC_MD_FLAG_ENABLE_COMPRESSION_ZLIB': 20,
    'MONGOC_MD_FLAG_ENABLE_SASL_GSSAPI': 21,
    'MONGOC_MD_FLAG_ENABLE_RES_NSEARCH': 22,
    'MONGOC_MD_FLAG_ENABLE_RES_NDESTROY': 23,
    'MONGOC_MD_FLAG_ENABLE_RES_NCLOSE': 24,
    'MONGOC_MD_FLAG_ENABLE_RES_SEARCH': 25,
    'MONGOC_MD_FLAG_ENABLE_DNSAPI': 26,
    'MONGOC_MD_FLAG_ENABLE_RDTSCP': 27,
    'MONGOC_MD_FLAG_HAVE_SCHED_GETCPU': 28,
    'MONGOC_MD_FLAG_ENABLE_SHM_COUNTERS': 29,
    'MONGOC_MD_FLAG_TRACE': 30,
    # `MONGOC_MD_FLAG_ENABLE_ICU` was accidentally removed in libmongoc 1.25.0-1.25.3.
    # If parsing a config-bitfield produced by libmongoc 1.25.0-1.25.3, use the version of `parse_handshake_cfg.py` from the git tag 1.25.0.
    'MONGOC_MD_FLAG_ENABLE_ICU': 31,
    'MONGOC_MD_FLAG_ENABLE_CLIENT_SIDE_ENCRYPTION': 32,
    'MONGOC_MD_FLAG_ENABLE_MONGODB_AWS_AUTH': 33,
    'MONGOC_MD_FLAG_ENABLE_SRV': 34,
    'MONGOC_MD_FLAG_HAVE_BCRYPT_PBKDF2': 35,
}


def main():
    flag_to_number = {s: 2**i for s, i in MD_FLAGS.items()}

    if len(sys.argv) < 2:
        print('Usage: python {0} config-bitfield'.format(sys.argv[0]))
        print('Example: python parse_handshake_cfg.py 0x3e65')
        return

    config_bitfield_string = sys.argv[1]
    config_bitfield_num = int(config_bitfield_string, 0)
    print('Decimal value: {}'.format(config_bitfield_num))

    for flag, num in flag_to_number.items():
        v = 'true' if config_bitfield_num & num else 'false'
        print('{:<50}: {}'.format(flag, v))


if __name__ == '__main__':
    main()



from shrub.v3.evg_build_variant import BuildVariant
from shrub.v3.evg_task import EvgTaskRef

from config_generator.etc.compile import generate_compile_tasks
from config_generator.etc.cse.compile import CompileCommon
from config_generator.etc.cse.test import generate_test_tasks

SSL = 'darwinssl'
TAG = f'cse-matrix-{SSL}'


# pylint: disable=line-too-long
# fmt: off
COMPILE_MATRIX = [
    ('macos-14-arm64', 'clang', None, ['cyrus']),
    ('macos-14',       'clang', None, ['cyrus']),
]

# QE (subset of CSFLE) requires 7.0+ and are skipped by "server" tasks.
TEST_MATRIX = [
    # Prefer macos-14-arm64 which is less resource-limited than macos-14. Provides 6.0+.
    ('macos-14-arm64', 'clang', None, 'cyrus', ['auth'], ['server', 'replica', 'sharded'], ['6.0', '7.0', '8.0', 'latest']),

    # Pre-6.0 coverage. Resource-limited: use sparingly.
    ('macos-14', 'clang', None, 'cyrus', ['auth'], ['sharded'], ['4.2', '4.4', '5.0']),
]
# fmt: on
# pylint: enable=line-too-long


class DarwinSSLCompileCommon(CompileCommon):
    ssl = 'DARWIN'


class SaslCyrusDarwinSSLCompile(DarwinSSLCompileCommon):
    name = 'cse-sasl-cyrus-darwinssl-compile'
    commands = DarwinSSLCompileCommon.compile_commands(sasl='CYRUS')


def functions():
    return SaslCyrusDarwinSSLCompile.defn()


def tasks():
    res = []

    SASL_TO_FUNC = {
        'cyrus': SaslCyrusDarwinSSLCompile,
    }

    MORE_TAGS = ['cse']

    res += generate_compile_tasks(SSL, TAG, SASL_TO_FUNC, COMPILE_MATRIX, MORE_TAGS)

    res += generate_test_tasks(SSL, TAG, TEST_MATRIX)

    return res


def variants():
    expansions = {
        'CLIENT_SIDE_ENCRYPTION': 'on',
    }

    return [
        BuildVariant(
            name=TAG,
            display_name=TAG,
            tasks=[EvgTaskRef(name=f'.{TAG}')],
            expansions=expansions,
        ),
    ]



# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import OrderedDict as OD

from evergreen_config_generator.variants import Variant

mobile_flags = (
    ' -DCMAKE_FIND_ROOT_PATH_MODE_LIBRARY=ONLY'
    ' -DCMAKE_FIND_ROOT_PATH_MODE_PACKAGE=ONLY'
    ' -DCMAKE_FIND_ROOT_PATH_MODE_PROGRAM=NEVER'
    ' -DCMAKE_FIND_ROOT_PATH_MODE_INCLUDE=ONLY'
)


def days(n: int) -> int:
    "Calculate the number of minutes in the given number of days"
    return n * 24 * 60


all_variants = [
    Variant(
        'abi-compliance-check',
        'ABI Compliance Check',
        ['ubuntu2204-small', 'ubuntu2204-medium', 'ubuntu2204-large'],
        ['abi-compliance-check'],
    ),
    Variant(
        'smoke',
        'Smoke Tests',
        'ubuntu2204-small',
        [
            'make-docs',
            'kms-divergence-check',
            'release-compile',
            'debug-compile-no-counters',
            'compile-tracing',
            'link-with-cmake',
            'link-with-cmake-ssl',
            'link-with-cmake-snappy',
            'verify-headers',
            OD([('name', 'link-with-cmake-mac'), ('distros', ['macos-14-arm64'])]),
            OD([('name', 'link-with-cmake-windows'), ('distros', ['windows-vsCurrent-large'])]),
            OD([('name', 'link-with-cmake-windows-ssl'), ('distros', ['windows-vsCurrent-large'])]),
            OD([('name', 'link-with-cmake-windows-snappy'), ('distros', ['windows-vsCurrent-large'])]),
            OD([('name', 'link-with-cmake-mingw'), ('distros', ['windows-vsCurrent-large'])]),
            OD([('name', 'link-with-pkg-config'), ('distros', ['ubuntu2204-large'])]),
            OD([('name', 'link-with-pkg-config-mac'), ('distros', ['macos-14-arm64'])]),
            'link-with-pkg-config-ssl',
            'link-with-bson',
            OD([('name', 'link-with-bson-windows'), ('distros', ['windows-vsCurrent-large'])]),
            OD([('name', 'link-with-bson-mac'), ('distros', ['macos-14-arm64'])]),
            OD([('name', 'link-with-bson-mingw'), ('distros', ['windows-vsCurrent-large'])]),
            'check-headers',
            'debug-compile-with-warnings',
            'install-libmongoc-after-libbson',
        ],
        {
            # Disable the MongoDB legacy shell download, which is not available in 5.0 for u22
            'SKIP_LEGACY_SHELL': '1'
        },
        tags=['pr-merge-gate'],
    ),
    Variant(
        'clang12-i686',
        'Clang 12 (i686) (Ubuntu 22.04)',
        'ubuntu2204-large',
        [
            'release-compile',
            'debug-compile-nosasl-nossl',
            '.debug-compile !.sspi .nossl .nosasl',
            '.latest .nossl .nosasl',
        ],
        {'CC': 'clang-12', 'MARCH': 'i686'},
    ),
    Variant(
        'gcc82rhel',
        'GCC 8.2 (RHEL 8.0)',
        'rhel80-test',
        [
            '.hardened',
            '.compression !.snappy !.zstd',
            'release-compile',
            'debug-compile-nosasl-nossl',
            'debug-compile-nosasl-openssl',
            'debug-compile-sasl-openssl',
            '.authentication-tests .openssl',
            '.latest .nossl',
        ],
        {'CC': 'gcc'},
    ),
    Variant(
        'gcc102',
        'GCC 10.2 (Debian 11.0)',
        'debian11-latest-large',
        ['release-compile', 'debug-compile-nosasl-nossl', '.latest .nossl'],
        {'CC': 'gcc'},
    ),
    Variant(
        'gcc11-i686',
        'GCC 11 (i686) (Ubuntu 22.04)',
        'ubuntu2204-large',
        ['release-compile', 'debug-compile-nosasl-nossl', '.latest .nossl .nosasl'],
        {'CC': 'gcc', 'MARCH': 'i686'},
    ),
    Variant(
        'gcc13',
        'GCC 13 (Ubuntu 24.04)',
        'ubuntu2404-large',
        [
            '.compression !.zstd',
            'debug-compile-nosrv',
            'release-compile',
            'debug-compile-nosasl-nossl',
            'debug-compile-sasl-openssl',
            'debug-compile-nosasl-openssl',
            '.authentication-tests .openssl',
            '.authentication-tests .asan',
            '.test-coverage',
            '.latest .nossl',
            'retry-true-latest-server',
            'test-dns-openssl',
            'test-dns-auth-openssl',
            'test-dns-loadbalanced-openssl',
        ],
        {'CC': 'gcc'},
    ),
    Variant(
        'darwin',
        '*Darwin, macOS (Apple LLVM)',
        'macos-14-arm64',
        [
            '.compression !.snappy',
            'release-compile',
            'debug-compile-nosasl-nossl',
            'debug-compile-nosrv',
            'debug-compile-sasl-darwinssl',
            'debug-compile-nosasl-nossl',
            '.authentication-tests .darwinssl',
            '.latest .nossl',
            'test-dns-darwinssl',
            'test-dns-auth-darwinssl',
            'debug-compile-lto',
            'debug-compile-lto-thin',
            'debug-compile-aws',
            'test-aws-openssl-regular-latest',
        ],
        {'CC': 'clang'},
    ),
    Variant(
        'darwin-intel',
        '*Darwin, Intel macOS (Apple LLVM)',
        'macos-14',
        [
            'debug-compile-aws',
            'debug-compile-rdtscp',
            'test-aws-openssl-regular-4.4',
        ],
        {'CC': 'clang'},
    ),
    Variant(
        'windows-2017-32',
        'Windows (i686) (VS 2017)',
        'windows-vsCurrent-large',
        ['debug-compile-nosasl-nossl', '.latest .nossl .nosasl'],
        {'CC': 'Visual Studio 15 2017'},
    ),
    Variant(
        'windows-2017',
        'Windows (VS 2017)',
        'windows-vsCurrent-large',
        [
            'release-compile',
            'debug-compile-nosasl-nossl',
            'debug-compile-nosasl-openssl',
            'debug-compile-sspi-winssl',
            'debug-compile-nosrv',
            '.latest .nossl',
            '.nosasl .latest .nossl',
            '.compression !.snappy !.zstd !.latest',
            'test-dns-winssl',
            'test-dns-auth-winssl',
            'debug-compile-aws',
            'test-aws-openssl-regular-4.4',
            'test-aws-openssl-regular-latest',
            # Authentication tests with OpenSSL on Windows are only run on the vs2017 variant.
            # Older vs variants fail to verify certificates against Atlas tests.
            '.authentication-tests .openssl !.sasl',
            '.authentication-tests .winssl',
        ],
        {'CC': 'Visual Studio 15 2017 Win64'},
    ),
    Variant(
        'windows-2022',
        'Windows (VS 2022)',
        'windows-vsCurrent-large',
        [
            'debug-compile-sspi-winssl',
            '.authentication-tests .winssl',
        ],
        {'CC': 'Visual Studio 17 2022 Win64'},
    ),
    Variant(
        'mingw-windows2016',
        'MinGW-W64 (Windows Server 2016)',
        'windows-vsCurrent-large',
        ['debug-compile-nosasl-nossl', '.latest .nossl .nosasl .server'],
        {'CC': 'gcc'},
    ),
    Variant(
        'rhel8-power',
        'Power (ppc64le) (RHEL 8)',
        'rhel8-power-large',
        [
            'release-compile',
            'debug-compile-nosasl-nossl',
            'debug-compile-sasl-openssl',
            '.latest .nossl',
            'test-dns-openssl',
        ],
        {'CC': 'gcc'},
        patchable=False,
        batchtime=days(1),
    ),
    Variant(
        'arm-ubuntu2204',
        '*ARM (aarch64) (Ubuntu 22.04)',
        'ubuntu2204-arm64-large',
        [
            '.compression !.snappy !.zstd',
            'release-compile',
            'debug-compile-nosasl-nossl',
            'debug-compile-nosasl-openssl',
            'debug-compile-sasl-openssl',
            '.authentication-tests .openssl',
            '.latest .nossl',
            'test-dns-openssl',
        ],
        {'CC': 'gcc'},
        batchtime=days(1),
    ),
    Variant(
        'zseries-rhel8',
        '*zSeries',
        'rhel8-zseries-large',
        [
            'release-compile',
            #      '.compression', --> TODO: waiting on ticket CDRIVER-3258
            'debug-compile-nosasl-nossl',
            'debug-compile-nosasl-openssl',
            'debug-compile-sasl-openssl',
            '.authentication-tests .openssl',
            '.latest .nossl',
        ],
        {'CC': 'gcc'},
        patchable=False,
        batchtime=days(1),
    ),
    # Run AWS tests for MongoDB 4.4 and 5.0 on Ubuntu 20.04. AWS setup scripts
    # expect Ubuntu 20.04+. MongoDB 4.4 and 5.0 are not available on 22.04.
    Variant(
        'aws-ubuntu2004',
        'AWS Tests (Ubuntu 20.04)',
        'ubuntu2004-small',
        [
            'debug-compile-aws',
            '.test-aws .4.4',
            '.test-aws .5.0',
        ],
        {'CC': 'clang'},
    ),
    Variant(
        'aws-ubuntu2204',
        'AWS Tests (Ubuntu 22.04)',
        'ubuntu2004-small',
        [
            'debug-compile-aws',
            '.test-aws .6.0',
            '.test-aws .7.0',
            '.test-aws .8.0',
            '.test-aws .latest',
        ],
        {'CC': 'clang'},
    ),
    Variant(
        'ocsp',
        'OCSP tests',
        'ubuntu2204-small',
        [
            OD([('name', 'debug-compile-nosasl-openssl')]),
            OD([('name', 'debug-compile-nosasl-darwinssl'), ('distros', ['macos-14-arm64'])]),
            OD([('name', 'debug-compile-nosasl-winssl'), ('distros', ['windows-vsCurrent-large'])]),
            OD([('name', '.ocsp-openssl')]),
            OD([('name', '.ocsp-darwinssl'), ('distros', ['macos-14-arm64'])]),
            OD([('name', '.ocsp-winssl'), ('distros', ['windows-vsCurrent-large'])]),
        ],
        {},
        batchtime=days(7),
        display_tasks=[
            {
                'name': 'ocsp-openssl',
                'execution_tasks': ['.ocsp-openssl'],
            },
            {
                'name': 'ocsp-darwinssl',
                'execution_tasks': ['.ocsp-darwinssl'],
            },
            {
                'name': 'ocsp-winssl',
                'execution_tasks': ['.ocsp-winssl'],
            },
        ],
    ),
    Variant(
        'packaging',
        'Linux Distro Packaging',
        'debian12-latest-small',
        [
            'debian-package-build',
            OD([('name', 'rpm-package-build'), ('distros', ['rhel90-arm64-small'])]),
        ],
        {},
        tags=['pr-merge-gate'],
    ),
    Variant(
        'versioned-api',
        'Versioned API Tests',
        'rhel8-latest-large',
        [
            'debug-compile-nosasl-openssl',
            'debug-compile-nosasl-nossl',
            '.versioned-api .5.0',
            '.versioned-api .6.0',
            '.versioned-api .7.0',
            '.versioned-api .8.0',
        ],
        {},
    ),
]



from __future__ import annotations

import functools
import re
from typing import Iterable, Literal, Mapping, NamedTuple, TypeVar

from shrub.v3.evg_build_variant import BuildVariant
from shrub.v3.evg_command import (
    BuiltInCommand,
    EvgCommandType,
    KeyValueParam,
    ec2_assume_role,
    expansions_update,
    subprocess_exec,
)
from shrub.v3.evg_task import EvgTask, EvgTaskRef

from config_generator.etc.function import Function

from ..etc.utils import all_possible

T = TypeVar('T')

_ENV_PARAM_NAME = 'MONGOC_EARTHLY_ENV'
_CC_PARAM_NAME = 'MONGOC_EARTHLY_C_COMPILER'
'The name of the EVG expansion for the Earthly c_compiler argument'


EnvKey = Literal[
    'u20',
    'u22',
    'almalinux8',
    'almalinux9',
    'almalinux10',
    'alpine3.19',
    'alpine3.20',
    'alpine3.21',
    'alpine3.22',
    'archlinux',
    'centos9',
    'centos10',
]
"Identifiers for environments. These correspond to special 'env.*' targets in the Earthfile."
CompilerName = Literal['gcc', 'clang']
'The name of the compiler program that is used for the build. Passed via --c_compiler to Earthly.'

# Other options: SSPI (Windows only), AUTO (not reliably test-able without more environments)
SASLOption = Literal['Cyrus', 'off']
'Valid options for the SASL configuration parameter'
TLSOption = Literal['OpenSSL', 'off']
"Options for the TLS backend configuration parameter (AKA 'ENABLE_SSL')"
CxxVersion = Literal['master', 'r4.1.0', 'none']
'C++ driver refs that are under CI test'

# A separator character, since we cannot use whitespace
_SEPARATOR = '\N{NO-BREAK SPACE}\N{BULLET}\N{NO-BREAK SPACE}'


def os_split(env: EnvKey) -> tuple[str, None | str]:
    """Convert the environment key into a pretty name+version pair"""
    match env:
        # Match 'alpine3.18' 'alpine53.123' etc.
        case alp if mat := re.match(r'alpine(\d+\.\d+)', alp):
            return ('Alpine', mat[1])
        case 'archlinux':
            return 'ArchLinux', None
        # Match 'u22', 'u20', 'u71' etc.
        case ubu if mat := re.match(r'u(\d\d)', ubu):
            return 'Ubuntu', f'{mat[1]}.04'
        # Match 'centos9', 'centos10', etc.
        case cent if mat := re.match(r'centos(\d+)', cent):
            return 'CentOS', f'{mat[1]}'
        # Match 'almalinux8', 'almalinux10', etc.
        case alm if mat := re.match(r'almalinux(\d+)', alm):
            return 'AlmaLinux', f'{mat[1]}'
        case _:
            raise ValueError(f'Failed to split OS env key {env=} into a name+version pair (unrecognized)')


class EarthlyVariant(NamedTuple):
    """
    Define a "variant" that runs under a set of Earthly parameters. These are
    turned into real EVG variants later on. The Earthly arguments are passed via
    expansion parameters.
    """

    env: EnvKey
    c_compiler: CompilerName

    @property
    def display_name(self) -> str:
        """The pretty name for this variant"""
        base: str
        match os_split(self.env):
            case name, None:
                base = name
            case name, version:
                base = f'{name} {version}'
        toolchain: str
        match self.c_compiler:
            case 'clang':
                toolchain = 'LLVM/Clang'
            case 'gcc':
                toolchain = 'GCC'
        return f'{base} ({toolchain})'

    @property
    def task_selector_tag(self) -> str:
        """
        The task tag that is used to select the tasks that want to run on this
        variant.
        """
        return f'{self.env}-{self.c_compiler}'

    @property
    def expansions(self) -> Mapping[str, str]:
        """
        Expansion values that are defined for the build variant that is generated
        from this object.
        """
        return {
            _CC_PARAM_NAME: self.c_compiler,
            _ENV_PARAM_NAME: self.env,
        }

    def as_evg_variant(self) -> BuildVariant:
        return BuildVariant(
            name=f'{self.task_selector_tag}',
            tasks=[EvgTaskRef(name=f'.{self.task_selector_tag}')],
            display_name=self.display_name,
            expansions=dict(self.expansions),
        )


class Configuration(NamedTuple):
    """
    Represent a complete set of argument values to give to the Earthly task
    execution. Each field name matches the ARG in the Earthfile.

    Adding/removing fields will add/remove dimensions on the task matrix.

    Some Earthly parameters are not encoded here, but are rather part of the variant (EarthlyVariant).
    """

    sasl: SASLOption
    tls: TLSOption
    test_mongocxx_ref: CxxVersion

    @property
    def suffix(self) -> str:
        return _SEPARATOR.join(f'{k}={v}' for k, v in self._asdict().items())


# Authenticate with DevProd-provided Amazon ECR instance to use as pull-through cache for DockerHub.
class DockerLoginAmazonECR(Function):
    name = 'docker-login-amazon-ecr'
    commands = [
        # Avoid inadvertently using a pre-existing and potentially conflicting Docker config.
        expansions_update(updates=[KeyValueParam(key='DOCKER_CONFIG', value='${workdir}/.docker')]),
        ec2_assume_role(role_arn='arn:aws:iam::901841024863:role/ecr-role-evergreen-ro'),
        subprocess_exec(
            binary='bash',
            command_type=EvgCommandType.SETUP,
            include_expansions_in_env=[
                'AWS_ACCESS_KEY_ID',
                'AWS_SECRET_ACCESS_KEY',
                'AWS_SESSION_TOKEN',
                'DOCKER_CONFIG',
            ],
            args=[
                '-c',
                'aws ecr get-login-password --region us-east-1 | docker login --username AWS --password-stdin 901841024863.dkr.ecr.us-east-1.amazonaws.com',
            ],
        ),
    ]


def task_filter(env: EarthlyVariant, conf: Configuration) -> bool:
    """
    Control which tasks are actually defined by matching on the platform and
    configuration values.
    """
    match env, conf:
        # Anything else: Allow it to run:
        case _:
            return True


def variants_for(config: Configuration) -> Iterable[EarthlyVariant]:
    """Get all Earthly variants that are not excluded for the given build configuration"""
    all_envs = all_possible(EarthlyVariant)
    allow_env_for_config = functools.partial(task_filter, conf=config)
    return filter(allow_env_for_config, all_envs)


def earthly_exec(
    *,
    kind: Literal['test', 'setup', 'system'],
    target: str,
    secrets: Mapping[str, str] | None = None,
    args: Mapping[str, str] | None = None,
) -> BuiltInCommand:
    """Create a subprocess_exec command that runs Earthly with the given arguments"""
    env: dict[str, str] = {k: v for k, v in (secrets or {}).items()}
    return subprocess_exec(
        './tools/earthly.sh',
        args=[
            # Use Amazon ECR as pull-through cache for DockerHub to avoid rate limits.
            '--buildkit-image=901841024863.dkr.ecr.us-east-1.amazonaws.com/dockerhub/earthly/buildkitd:v0.8.3',
            *(f'--secret={k}' for k in (secrets or ())),
            f'+{target}',
            # Use Amazon ECR as pull-through cache for DockerHub to avoid rate limits.
            '--default_search_registry=901841024863.dkr.ecr.us-east-1.amazonaws.com/dockerhub',
            *(f'--{arg}={val}' for arg, val in (args or {}).items()),
        ],
        command_type=EvgCommandType(kind),
        include_expansions_in_env=['DOCKER_CONFIG'],
        env=env if env else None,
        working_dir='mongoc',
    )


def earthly_task(
    *,
    name: str,
    targets: Iterable[str],
    config: Configuration,
) -> EvgTask | None:
    """
    Create an EVG task which executes earthly using the given parameters. If this
    function returns `None`, then the task configuration is excluded from executing
    and no task should be defined.
    """
    # Attach tags to the task to allow build variants to select
    # these tasks by the environment of that variant.
    env_tags = sorted(e.task_selector_tag for e in sorted(variants_for(config)))
    if not env_tags:
        # All environments have been excluded for this configuration. This means
        # the task itself should not be run:
        return
    # Generate the build-arg arguments based on the configuration options. The
    # NamedTuple field names must match with the ARG keys in the Earthfile!
    earthly_args = config._asdict()
    earthly_args |= {
        # Add arguments that come from parameter expansions defined in the build variant
        'env': f'${{{_ENV_PARAM_NAME}}}',
        'c_compiler': f'${{{_CC_PARAM_NAME}}}',
    }
    return EvgTask(
        name=name,
        commands=[
            DockerLoginAmazonECR.call(),
            # First, just build the "env-warmup" which will prepare the build environment.
            # This won't generate any output, but allows EVG to track it as a separate build step
            # for timing and logging purposes. The subequent build step will cache-hit the
            # warmed-up build environments.
            earthly_exec(
                kind='setup',
                target='env-warmup',
                args=earthly_args,
            ),
            # Now execute the main tasks:
            earthly_exec(
                kind='test',
                target='run',
                # The "targets" arg is for +run to specify which targets to run
                args={'targets': ' '.join(targets)} | earthly_args,
            ),
        ],  # type: ignore (The type annots on `commands` is wrong)
        tags=['earthly', 'pr-merge-gate', *env_tags],
        run_on=CONTAINER_RUN_DISTROS,
    )


CONTAINER_RUN_DISTROS = [
    'amazon2',
    'debian11-latest-large',
    'debian12-latest-large',
    'ubuntu2204-large',
    'ubuntu2404-large',
]


def functions():
    return DockerLoginAmazonECR.defn()


def tasks() -> Iterable[EvgTask]:
    for conf in all_possible(Configuration):
        # test-example is a target in all configurations
        targets = ['test-example']

        # test-cxx-driver is only a target in configurations with specified mongocxx versions
        if conf.test_mongocxx_ref != 'none':
            targets.append('test-cxx-driver')

        task = earthly_task(
            name=f'check:{conf.suffix}',
            targets=targets,
            config=conf,
        )
        if task is not None:
            yield task

    yield EvgTask(
        name='verify-headers',
        commands=[
            DockerLoginAmazonECR.call(),
            earthly_exec(kind='test', target='verify-headers'),
        ],
        tags=['pr-merge-gate'],
        run_on=CONTAINER_RUN_DISTROS,
    )


def variants() -> Iterable[BuildVariant]:
    yield from (ev.as_evg_variant() for ev in all_possible(EarthlyVariant))



"""
This script generates a C source file containing test cases for BSON validation
and iteration.

Run this script with Python 3.12+, and pipe the output into a file.

This script takes no command-line arguments.
"""

# /// script
# requires-python = ">=3.12"
# dependencies = []
# ///

import argparse
import enum
import json
import re
import struct
import textwrap
from dataclasses import dataclass
from typing import Iterable


class Tag(enum.Enum):
    """BSON type tag byte values"""

    EOD = 0
    Double = 1
    UTF8 = 2
    Document = 3
    Array = 4
    Binary = 5
    Undefined = 6
    OID = 7
    Boolean = 8
    Datetime = 9
    Null = 10
    Regex = 11
    DBPointer = 12
    Code = 13
    Symbol = 14
    CodeWithScope = 15
    Int32 = 16
    Timestamp = 17
    Int64 = 18
    Decimal128 = 19
    MinKey = 0xFF
    MaxKey = 0x7F


type _ByteIter = bytes | Iterable[_ByteIter]
"""A set of bytes, or an iterable that yields more sets of bytes"""


def flatten_bytes(data: _ByteIter) -> bytes:
    """Flatten a (recursive) iterator of bytes into a single bytes object"""
    match data:
        case bytes(data):
            return data
        case it:
            return b''.join(map(flatten_bytes, it))


def i32le(i: int) -> bytes:
    """Encode an integer as a 32-bit little-endian integer"""
    return struct.pack('<i', i)


def i64le(i: int) -> bytes:
    """Encode an integer as a 64-bit little-endian integer"""
    return struct.pack('<q', i)


def f64le(f: float) -> bytes:
    """Encode a float as a 64-bit little-endian float"""
    return struct.pack('<d', f)


def doc(*data: _ByteIter) -> bytes:
    """Add a BSON document header a null terminator to a set of bytes"""
    flat = flatten_bytes(data)
    # +5 for the null terminator and the header bytes
    hdr = i32le(len(flat) + 5)
    return hdr + flat + b'\0'


def code_with_scope(code: str, doc: _ByteIter) -> bytes:
    """Create a BSON code-with-scope object with appropriate header"""
    s = string(code)
    doc = flatten_bytes(doc)
    # +4 to include the length prefix too
    len_prefix = i32le(len(s) + len(doc) + 4)
    return len_prefix + s + doc


def elem(key: str | _ByteIter, tag: int | Tag, *bs: _ByteIter) -> bytes:
    """Add a BSON element header to a set of bytes"""
    if isinstance(tag, Tag):
        tag = tag.value
    return bytes([tag]) + cstring(key) + flatten_bytes(bs)


def binary(subtype: int, *bs: _ByteIter) -> bytes:
    """
    Create a BSON binary object with appropriate header and subtype tag byte.
    """
    flat = flatten_bytes(bs)
    st = bytes([subtype])
    return i32le(len(flat)) + st + flat


def cstring(s: str | _ByteIter) -> bytes:
    """Encode a string as UTF-8 and add a null terminator"""
    match s:
        case str(s):
            return cstring(s.encode('utf-8'))
        case bs:
            bs = flatten_bytes(bs)
            return bs + b'\0'


def string(s: str | _ByteIter) -> bytes:
    """Add a length header and null terminator to a UTF-8 string"""
    cs = cstring(s)
    # Length header includes the null terminator
    hdr = i32le(len(cs))
    return hdr + cs


def utf8elem(key: str | _ByteIter, s: str | _ByteIter) -> bytes:
    """Create a valid UTF-8 BSON element for the given string"""
    return elem(key, Tag.UTF8, string(s))


@dataclass(frozen=True)
class ErrorInfo:
    """
    Information about an expected validation error
    """

    code: str
    """Spellling of the error code to be expected"""
    message: str
    """The expected error message"""
    offset: int
    """The expected error offset"""


@dataclass(frozen=True)
class TestCase:
    """
    Defines a single validation test case.
    """

    name: str
    """The name of the test case, as displayed in test runners, which will have a "/bson/validate" prefix"""
    data: bytes
    """The bytes that will be injested by `bson_init_static` to form the document to be validated"""
    description: str | None
    """A plaintext description of the test case and what it actually does. Rendered as a comment."""
    flags: str = '0'
    """Spelling of the flags argument passed to the validation API"""
    error: ErrorInfo = ErrorInfo('0', '', 0)
    """Expected error, if any"""

    @property
    def fn_name(self) -> str:
        """Get a C identifier function name for this test case"""
        return '_test_case_' + re.sub(r'[^\w]', '_', self.name).lower()


def fmt_byte(n: int) -> str:
    """
    Format an octet value for C code. Will emit a char literal if certain ASCII,
    otherwise an integer literal.
    """
    match n:
        case 0:
            return '0'
        case a if re.match(r'[a-zA-Z0-9.$-]', chr(a)):
            return f"'{chr(a)}'"
        case a if a < 10:
            return str(a)
        case n:
            return f'0x{n:0>2x}'


GENERATED_NOTE = '// ! This code is GENERATED! Do not edit it directly!'

HEADER = rf"""{GENERATED_NOTE}
// clang-format off

#include <bson/bson.h>

#include <mlib/test.h>

#include <TestSuite.h>
"""


def generate(case: TestCase) -> Iterable[str]:
    """
    Generate the lines of a test case function.
    """
    # A comment header
    yield f'{GENERATED_NOTE}\n'
    yield f'// Case: {case.name}\n'
    # The function head
    yield f'static inline void {case.fn_name}(void) {{\n'
    # If we have a description, emit that in a block comment
    if case.description:
        yield '  /**\n'
        lines = textwrap.dedent(case.description).strip().splitlines()
        yield from (f'   * {ln}\n' for ln in lines)
        yield '   */\n'
    # Emit the byte array literal
    yield '  const uint8_t bytes[] = {\n'
    yield '\n'.join(
        textwrap.wrap(
            ', '.join(map(fmt_byte, case.data)),
            subsequent_indent=' ' * 4,
            initial_indent=' ' * 4,
            width=80,
        )
    )
    yield '\n  };\n'
    yield from [
        # Initialize a BSON doc that points to the byte array
        '  bson_t doc;\n',
        '  mlib_check(bson_init_static(&doc, bytes, sizeof bytes));\n',
        # The error object to be filled
        '  bson_error_t error = {0};\n',
        # The error offset. Expected to be reset to zero on success.
        '  size_t offset = 999999;\n'
        # Do the actual validation:
        f'  const bool is_valid = bson_validate_with_error_and_offset(&doc, {case.flags}, &offset, &error);\n',
    ]
    is_error = case.error.code != '0'
    yield from [
        '  mlib_check(!is_valid);\n' if is_error else '  ASSERT_OR_PRINT(is_valid, error);\n',
        f'  mlib_check(error.code, eq, {case.error.code});\n',
        f'  mlib_check(error.message, str_eq, {json.dumps(case.error.message)});\n',
        f'  mlib_check(offset, eq, {case.error.offset});\n' if is_error else '',
    ]
    yield '}\n'


def corruption_at(off: int) -> ErrorInfo:
    """
    Generate an ErrorInfo to expect a message of "corrupt BSON" at the given
    byte offset.

    Note that this won't match if the error message is something other
    than "corrupt BSON".
    """
    return ErrorInfo(BSON_VALIDATE_CORRUPT, 'corrupt BSON', off)


BSON_VALIDATE_CORRUPT = 'BSON_VALIDATE_CORRUPT'
BSON_VALIDATE_DOLLAR_KEYS = 'BSON_VALIDATE_DOLLAR_KEYS'
BSON_VALIDATE_DOT_KEYS = 'BSON_VALIDATE_DOT_KEYS'
BSON_VALIDATE_EMPTY_KEYS = 'BSON_VALIDATE_EMPTY_KEYS'
BSON_VALIDATE_UTF8 = 'BSON_VALIDATE_UTF8'
BSON_VALIDATE_UTF8_ALLOW_NULL = 'BSON_VALIDATE_UTF8_ALLOW_NULL'
MSG_EXPECTED_ID_FOLLOWING_REF = 'Expected an $id element following $ref'


def disallowed_key(char: str, k: str) -> str:
    return f'Disallowed \'{char}\' in element key: "{k}"'


# d888888b d88888b .d8888. d888888b       .o88b.  .d8b.  .d8888. d88888b .d8888.
# `~~88~~' 88'     88'  YP `~~88~~'      d8P  Y8 d8' `8b 88'  YP 88'     88'  YP
#    88    88ooooo `8bo.      88         8P      88ooo88 `8bo.   88ooooo `8bo.
#    88    88~~~~~   `Y8b.    88         8b      88~~~88   `Y8b. 88~~~~~   `Y8b.
#    88    88.     db   8D    88         Y8b  d8 88   88 db   8D 88.     db   8D
#    YP    Y88888P `8888Y'    YP          `Y88P' YP   YP `8888Y' Y88888P `8888Y'

CASES: list[TestCase] = [
    TestCase(
        'empty',
        doc(),
        """Test a simple empty document object.""",
    ),
    TestCase(
        'bad-element',
        doc(b'f'),
        'The element content is not valid',
        error=corruption_at(6),
    ),
    TestCase(
        'invalid-type',
        doc(elem('foo', 0xE, b'foo')),
        """The type tag "0x0e" is not a valid type""",
        error=corruption_at(9),
    ),
    TestCase(
        'key/invalid/accept',
        doc(
            utf8elem('a', 'b'),
            utf8elem(b'foo\xffbar', 'baz'),
            utf8elem('c', 'd'),
        ),
        """
        The element key contains an invalid UTF-8 byte, but we accept it
        because we aren't doing UTF-8 validation.
        """,
    ),
    TestCase(
        'key/invalid/reject',
        doc(
            utf8elem('a', 'b'),
            elem(b'foo\xffbar', Tag.UTF8, string('baz')),
            utf8elem('c', 'd'),
        ),
        """
        The element key is not valid UTF-8 and we reject it when we do UTF-8
        validation.
        """,
        flags=BSON_VALIDATE_UTF8,
        error=ErrorInfo(BSON_VALIDATE_UTF8, 'Text element is not valid UTF-8', 13),
    ),
    TestCase(
        'key/empty/accept',
        doc(utf8elem('', 'string')),
        """
        The element has an empty string key, and we accept this.
        """,
    ),
    TestCase(
        'key/empty/reject',
        doc(
            utf8elem('a', 'b'),
            utf8elem('', 'string'),
        ),
        """
        The element has an empty key, and we can reject it.
        """,
        flags=BSON_VALIDATE_EMPTY_KEYS,
        error=ErrorInfo(BSON_VALIDATE_EMPTY_KEYS, 'Element key cannot be an empty string', 13),
    ),
    TestCase(
        'key/empty/accept-if-absent',
        doc(utf8elem('foo', 'bar')),
        """
        We are checking for empty keys, and accept if they are absent.
        """,
        flags=BSON_VALIDATE_EMPTY_KEYS,
    ),
    TestCase(
        'key/dot/accept',
        doc(utf8elem('foo.bar', 'baz')),
        """
        The element key has an ASCII dot, and we accept this since we don't
        ask to validate it.
        """,
        flags=BSON_VALIDATE_EMPTY_KEYS,
    ),
    TestCase(
        'key/dot/reject',
        doc(utf8elem('a', 'b'), utf8elem('foo.bar', 'baz')),
        """
        The element has an ASCII dot, and we reject it when we ask to validate
        it.
        """,
        flags=BSON_VALIDATE_DOT_KEYS,
        error=ErrorInfo(BSON_VALIDATE_DOT_KEYS, disallowed_key('.', 'foo.bar'), 13),
    ),
    TestCase(
        'key/dot/accept-if-absent',
        doc(utf8elem('foo', 'bar')),
        """
        We are checking for keys with dot '.', and accept if they are absent.
        """,
        flags=BSON_VALIDATE_DOT_KEYS,
    ),
    TestCase(
        'key/dollar/accept',
        doc(utf8elem('a', 'b'), utf8elem('$foo', 'bar')),
        """
        We can accept an element key that starts with a dollar '$' sign.
        """,
    ),
    TestCase(
        'key/dollar/reject',
        doc(utf8elem('a', 'b'), utf8elem('$foo', 'bar')),
        """
        We can reject an element key that starts with a dollar '$' sign.
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
        error=ErrorInfo(BSON_VALIDATE_DOLLAR_KEYS, disallowed_key('$', '$foo'), 13),
    ),
    TestCase(
        'key/dollar/accept-in-middle',
        doc(utf8elem('foo$bar', 'baz')),
        """
        This contains a element key "foo$bar", but we don't reject this, as we
        only care about keys that *start* with dollars.
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
    ),
    TestCase(
        'key/dollar/accept-if-absent',
        doc(utf8elem('foo', 'bar')),
        """
        We are validating for dollar-keys, and we accept because this document
        doesn't contain any such keys.
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
    ),
    TestCase(
        'utf8/simple',
        doc(utf8elem('string', 'some string')),
        'Simple UTF-8 string element',
    ),
    TestCase(
        'utf8/missing-null',
        doc(elem('a', Tag.UTF8, i32le(4), b'abcd')),
        """
        The UTF-8 element "a" contains 4 characters and declares its length of 4,
        but the fourth character is supposed to be a null terminator. In this case,
        it is the letter 'd'.
        """,
        error=corruption_at(14),
    ),
    TestCase(
        'utf8/length-zero',
        doc(elem('', Tag.UTF8, i32le(0), b'\0')),
        'UTF-8 string length must always be at least 1 for the null terminator',
        error=corruption_at(6),
    ),
    TestCase(
        'utf8/length-too-short',
        doc(elem('', Tag.UTF8, i32le(3), b'bar\0')),
        'UTF-8 string is three chars and a null terminator, but the declared length is 3 (should be 4)',
        error=corruption_at(12),
    ),
    TestCase(
        'utf8/header-too-large',
        doc(elem('foo', Tag.UTF8, b'\xff\xff\xff\xffbar\0')),
        """
        Data { "foo": "bar" } but the declared length of "bar" is way too large.
        """,
        error=corruption_at(9),
    ),
    TestCase(
        'utf8/valid',
        doc(elem('foo', Tag.UTF8, string('abcd'))),
        """
        Validate a valid UTF-8 string with UTF-8 validation enabled.
        """,
        flags=BSON_VALIDATE_UTF8,
    ),
    TestCase(
        'utf8/invalid/accept',
        doc(utf8elem('foo', b'abc\xffd')),
        """
        Validate an invalid UTF-8 string, but accept invalid UTF-8.
        """,
    ),
    TestCase(
        'utf8/invalid/reject',
        doc(utf8elem('foo', b'abc\xffd')),
        """
        Validate an invalid UTF-8 string, and expect rejection.
        """,
        flags=BSON_VALIDATE_UTF8,
        error=ErrorInfo(BSON_VALIDATE_UTF8, 'Text element is not valid UTF-8', 4),
    ),
    TestCase(
        'utf8/valid-with-null/accept-1',
        doc(utf8elem('foo', b'abc\x00123')),
        """
        This is a valid UTF-8 string that contains a null character. We accept
        it because we don't do UTF-8 validation.
        """,
    ),
    TestCase(
        'utf8/valid-with-null/accept-2',
        doc(utf8elem('foo', b'abc\x00123')),
        """
        This is a valid UTF-8 string that contains a null character. We allow
        it explicitly when we request UTF-8 validation.
        """,
        flags=f'{BSON_VALIDATE_UTF8} | {BSON_VALIDATE_UTF8_ALLOW_NULL}',
    ),
    TestCase(
        'utf8/valid-with-null/reject',
        doc(utf8elem('foo', b'abc\x00123')),
        """
        This is a valid UTF-8 string that contains a null character. We reject
        this because we don't pass BSON_VALIDATE_UTF8_ALLOW_NULL.
        """,
        flags=BSON_VALIDATE_UTF8,
        error=ErrorInfo(BSON_VALIDATE_UTF8_ALLOW_NULL, 'UTF-8 string contains a U+0000 (null) character', 4),
    ),
    TestCase(
        'utf8/overlong-null/accept-1',
        doc(utf8elem('foo', b'abc\xc0\x80123')),
        """
        This is an *invalid* UTF-8 string, and contains an overlong null. We should
        accept it because we aren't doing UTF-8 validation.
        """,
    ),
    TestCase(
        'utf8/overlong-null/accept-2',
        doc(utf8elem('foo', b'abc\xc0\x80123')),
        """
        ! NOTE: overlong-null: This test relies on our UTF-8 validation accepting the `c0 80` sequence

        This is an *invalid* UTF-8 string, because it contains an overlong null
        "0xc0 0x80". Despite being invalid, we accept it because our current UTF-8
        validation considers the overlong null to be a valid encoding for the null
        codepoint (it isn't, but changing it would be a breaking change).

        If/when UTF-8 validation is changed to reject overlong null, then this
        test should change to expect rejection the invalid UTF-8.
        """,
        flags=f'{BSON_VALIDATE_UTF8} | {BSON_VALIDATE_UTF8_ALLOW_NULL}',
    ),
    TestCase(
        'utf8/overlong-null/reject',
        doc(utf8elem('foo', b'abc\xc0\x80123')),
        """
        ! NOTE: overlong-null: This test relies on our UTF-8 validation accepting the `c0 80` sequence

        This is an *invalid* UTF-8 string, because it contains an overlong null
        character. Our UTF-8 validator wrongly accepts overlong null as a valid
        UTF-8 sequence. This test fails because we disallow null codepoints, not
        because the UTF-8 is invalid, and the error message reflects that.

        If/when UTF-8 validation is changed to reject overlong null, then the
        expected error code and error message for this test should change.
        """,
        flags=BSON_VALIDATE_UTF8,
        error=ErrorInfo(BSON_VALIDATE_UTF8_ALLOW_NULL, 'UTF-8 string contains a U+0000 (null) character', 4),
    ),
    TestCase(
        'utf8-key/invalid/accept',
        doc(utf8elem(b'abc\xffdef', 'bar')),
        """
        The element key is not valid UTf-8, but we accept it if we don't do
        UTF-8 validation.
        """,
    ),
    TestCase(
        'utf8-key/invalid/reject',
        doc(utf8elem(b'abc\xffdef', 'bar')),
        """
        The element key is not valid UTF-8, and we reject it when we requested
        UTF-8 validation.
        """,
        flags=BSON_VALIDATE_UTF8,
        error=ErrorInfo(BSON_VALIDATE_UTF8, 'Text element is not valid UTF-8', 4),
    ),
    TestCase(
        'utf8-key/overlong-null/reject',
        doc(utf8elem(b'abc\xc0\x80def', 'bar')),
        """
        ! NOTE: overlong-null: This test relies on our UTF-8 validation accepting the `c0 80` sequence

        The element key is invalid UTF-8 because it contains an overlong null. We accept the
        overlong null as a valid encoding of U+0000, but we reject the key because
        we disallow null in UTF-8 strings.

        If/when UTF-8 validation is changed to reject overlong null, then the
        expected error code and error message for this test should change.
        """,
        flags=BSON_VALIDATE_UTF8,
        error=ErrorInfo(BSON_VALIDATE_UTF8_ALLOW_NULL, 'UTF-8 string contains a U+0000 (null) character', 4),
    ),
    TestCase(
        'utf8-key/overlong-null/accept',
        doc(utf8elem(b'abc\xc0\x80def', 'bar')),
        """
        ! NOTE: overlong-null: This test relies on our UTF-8 validation accepting the `c0 80` sequence

        The element key is invalid UTF-8 because it contains an overlong null. We accept the
        overlong null as a valid encoding of U+0000, and we allow it in an element key because
        we pass ALLOW_NULL

        If/when UTF-8 validation is changed to reject overlong null, then this
        test case should instead reject the key string as invalid UTF-8.
        """,
        flags=f'{BSON_VALIDATE_UTF8} | {BSON_VALIDATE_UTF8_ALLOW_NULL}',
    ),
    TestCase(
        'array/empty',
        doc(elem('array', Tag.Array, doc())),
        'Simple empty array element',
    ),
    TestCase(
        'array/simple',
        doc(
            elem(
                'array',
                Tag.Array,
                doc(
                    elem('0', Tag.Int32, i32le(42)),
                    elem('1', Tag.Int32, i32le(1729)),
                    elem('2', Tag.Int32, i32le(-8)),
                ),
            )
        ),
        'Simple array element of integers',
    ),
    TestCase(
        'array/invalid-element',
        doc(
            elem(
                'array',
                Tag.Array,
                doc(
                    elem('0', Tag.Int32, i32le(42)),
                    elem('1', Tag.Int32, i32le(1729)[-1:]),  # Truncated
                    elem('2', Tag.Int32, i32le(-8)),
                ),
            )
        ),
        'Simple array element of integers, but one element is truncated',
        error=corruption_at(34),
    ),
    TestCase(
        'array/invalid-element-check-offset',
        doc(
            elem(
                'array-shifted',
                Tag.Array,
                doc(
                    elem('0', Tag.Int32, i32le(42)),
                    elem('1', Tag.Int32, i32le(1729)[-1:]),  # Truncated
                    elem('2', Tag.Int32, i32le(-8)),
                ),
            )
        ),
        """
        This is the same as the array/invalid-element test, but with a longer
        key string on the parent array. This is to check that the error offset
        is properly adjusted for the additional characters.
        """,
        error=corruption_at(42),
    ),
    TestCase(
        'symbol/simple',
        doc(elem('symbol', Tag.Symbol, string('void 0;'))),
        """
        A simple document: { symbol: Symbol("void 0;") }
        """,
    ),
    TestCase(
        'symbol/invalid-utf8/accept',
        doc(elem('symbol', Tag.Symbol, string(b'void\xff 0;'))),
        """
        A simple symbol document, but the string contains invalid UTF-8
        """,
    ),
    TestCase(
        'symbol/invalid-utf8/reject',
        doc(elem('symbol', Tag.Symbol, string(b'void\xff 0;'))),
        """
        A simple symbol document, but the string contains invalid UTF-8
        """,
        flags=BSON_VALIDATE_UTF8,
        error=ErrorInfo(BSON_VALIDATE_UTF8, 'Text element is not valid UTF-8', 4),
    ),
    TestCase(
        'symbol/length-zero',
        doc(b'\x0e\0' + i32le(0) + b'\0'),
        'Symbol string length must always be at least 1 for the null terminator',
        error=corruption_at(6),
    ),
    TestCase(
        'symbol/length-too-short',
        doc(b'\x0e\0' + i32le(3) + b'bar\0'),
        """
        Symbol string is three chars and a null terminator, but the declared
        length is 3 (should be 4)
        """,
        error=corruption_at(12),
    ),
    TestCase(
        'code/simple',
        doc(elem('code', Tag.Code, string('void 0;'))),
        """
        A simple document: { code: Code("void 0;") }
        """,
    ),
    TestCase(
        'code/invalid-utf8/accept',
        doc(elem('code', Tag.Code, string(b'void\xff 0;'))),
        """
        A simple code document, but the string contains invalid UTF-8
        """,
    ),
    TestCase(
        'code/invalid-utf8/reject',
        doc(elem('code', Tag.Code, string(b'void\xff 0;'))),
        """
        A simple code document, but the string contains invalid UTF-8
        """,
        flags=BSON_VALIDATE_UTF8,
        error=ErrorInfo(BSON_VALIDATE_UTF8, 'Text element is not valid UTF-8', 4),
    ),
    TestCase(
        'code/length-zero',
        doc(elem('code', Tag.Code, i32le(0), b'\0')),
        'Code string length must always be at least 1 for the null terminator',
        error=corruption_at(10),
    ),
    TestCase(
        'code/length-too-short',
        doc(elem('code', Tag.Code, i32le(3), b'bar\0')),
        'Code string is three chars and a null terminator, but the declared length is 3 (should be 4)',
        error=corruption_at(16),
    ),
    # Code w/ scope
    TestCase(
        'code-with-scope/simple',
        doc(elem('foo', Tag.CodeWithScope, code_with_scope('void 0;', doc()))),
        'A simple valid code-with-scope element',
    ),
    TestCase(
        'code-with-scope/invalid-code-length-zero',
        doc(
            elem(
                '',
                Tag.CodeWithScope,
                i32le(10),
                b'\0\0\0\0',  # strlen
                b'\0',  # code
                doc(),  # scope
            )
        ),
        """
        Data { "": CodeWithScope("", {}) }, but the code string length is zero, when
        it must be at least 1
        """,
        error=corruption_at(6),
    ),
    TestCase(
        'code-with-scope/invalid-code-length-too-large',
        doc(
            elem(
                '',
                Tag.CodeWithScope,
                i32le(10),
                b'\xff\xff\xff\xff',  # strlen (too big)
                b'\0',
                doc(),  # Scope
            )
        ),
        """
        Data { "": CodeWithScope("", {}) }, but the code string length is way too large
        """,
        error=corruption_at(6),
    ),
    TestCase(
        'code-with-scope/invalid-scope',
        doc(elem('foo', Tag.CodeWithScope, code_with_scope('void 0;', doc()[:-1]))),
        'A code-with-scope element, but the scope document is corrupted',
        error=corruption_at(13),
    ),
    TestCase(
        'code-with-scope/empty-key-in-scope',
        doc(
            elem(
                'code',
                Tag.CodeWithScope,
                code_with_scope(
                    'void 0;',
                    doc(
                        elem('obj', Tag.Document, doc(utf8elem('', 'string'))),
                    ),
                ),
            )
        ),
        """
        A code-with-scope element. The scope itself contains empty keys within
        objects, and we ask to reject empty keys. But the scope document should
        be treated as an opaque closure, so our outer validation rules do not
        apply.
        """,
        flags=BSON_VALIDATE_EMPTY_KEYS,
    ),
    TestCase(
        'code-with-scope/corrupt-scope',
        doc(
            elem(
                'code',
                Tag.CodeWithScope,
                code_with_scope(
                    'void 0;',
                    doc(
                        elem(
                            'foo',
                            Tag.UTF8,
                            i32le(0),  # Invalid string length
                            b'\0',
                        )
                    ),
                ),
            )
        ),
        'A code-with-scope element, but the scope contains corruption',
        error=ErrorInfo(BSON_VALIDATE_CORRUPT, 'Error in scope document for element "code": corrupt BSON', offset=13),
    ),
    TestCase(
        'code-with-scope/corrupt-scope-2',
        doc(
            elem(
                'code',
                Tag.CodeWithScope,
                code_with_scope(
                    'void 0;',
                    doc(
                        elem(
                            'foo',
                            Tag.UTF8,
                            b'\xff\xff\xff\xff',  # Invalid string length
                            b'\0',
                        )
                    ),
                ),
            )
        ),
        'A code-with-scope element, but the scope contains corruption',
        error=ErrorInfo(BSON_VALIDATE_CORRUPT, 'Error in scope document for element "code": corrupt BSON', offset=13),
    ),
    TestCase(
        'regex/simple',
        doc(elem('regex', Tag.Regex, b'1234\0gi\0')),
        """
        Simple document: { regex: Regex("1234", "gi") }
        """,
    ),
    TestCase(
        'regex/invalid-opts',
        doc(elem('regex', Tag.Regex, b'foo\0bar')),
        """
        A regular expression element with missing null terminator. The main
        option string "foo" has a null terminator, but the option component "bar"
        does not have a null terminator. A naive parse will see the doc's null
        terminator as the null terminator for the options string, but that's
        invalid!
        """,
        error=corruption_at(18),
    ),
    TestCase(
        'regex/double-null',
        doc(elem('regex', Tag.Regex, b'foo\0bar\0\0')),
        """
        A regular expression element with an extra null terminator. Since regex
        is delimited by its null terminator, the iterator will stop early before
        the actual EOD.
        """,
        error=corruption_at(21),
    ),
    TestCase(
        'regex/invalid-utf8/accept',
        doc(elem('regex', Tag.Regex, b'foo\xffbar\0gi\0')),
        """
        A regular expression that contains invalid UTF-8.
        """,
    ),
    TestCase(
        'regex/invalid-utf8/reject',
        doc(elem('regex', Tag.Regex, b'foo\xffbar\0gi\0')),
        """
        A regular expression that contains invalid UTF-8.
        """,
        flags=BSON_VALIDATE_UTF8,
        error=ErrorInfo(BSON_VALIDATE_UTF8, 'Text element is not valid UTF-8', 4),
    ),
    TestCase(
        'regex/invalid-utf8/accept-if-absent',
        doc(elem('regex', Tag.Regex, b'foo\0gi\0')),
        """
        A regular valid UTf-8 regex. We check for invalid UTf-8, and accept becaues
        the regex is fine.
        """,
        flags=BSON_VALIDATE_UTF8,
    ),
    TestCase(
        'dbpointer/string-length-zero',
        doc(
            elem(
                'foo',
                Tag.DBPointer,
                i32le(0),  # String length (invalid)
                b'\0',  # Empty string
                b'\x52\x59\xb5\x6a\xfa\x5b\xd8\x41\xd6\x58\x5d\x99',  # OID
            )
        ),
        """
        Document { "foo": DBPointer("", <oid>) }, but the length header on the inner
        string is zero, when it must be at least 1.
        """,
        error=corruption_at(9),
    ),
    TestCase(
        'dbpointer/string-length-too-big',
        doc(
            elem(
                'foo',
                Tag.DBPointer,
                b'\xff\xff\xff\xff',  # String length  (invalid)
                b'foobar\0',  # Simple string
                b'\x52\x59\xb5\x6a\xfa\x5b\xd8\x41\xd6\x58\x5d\x99',  # OID
            )
        ),
        """
        Document { "foo": DBPointer("foobar", <oid>) }, but the length header on the inner
        string is far too large
        """,
        error=corruption_at(9),
    ),
    TestCase(
        'dbpointer/truncated',
        doc(
            utf8elem('a', 'b'),
            elem(
                'foo',
                Tag.DBPointer,
                i32le(7),  # 7 bytes, bleeding into the null terminator
                b'foobar',  # Simple string, missing a null terminator.
                b'\x00\x59\xb5\x6a\xfa\x5b\xd8\x41\xd6\x58\x5d\x99',  # OID
            ),
            utf8elem('a', 'b'),
        ),
        """
        Document { "foo": DBPointer("foobar", <oid>) }, but the length header on
        the string is one byte too large, causing it to use the first byte of the
        OID as the null terminator. This should fail when iterating.
        """,
        error=corruption_at(43),
    ),
    TestCase(
        'dbpointer/missing-null',
        doc(
            elem(
                'foo',
                Tag.DBPointer,
                i32le(4),
                b'abcd',  # Missing null terminator
                b'\x52\x59\xb5\x6a\xfa\x5b\xd8\x41\xd6\x58\x5d\x99',  # OID
            )
        ),
        """
        Document { "foo": DBPointer("abcd", <oid>) }, the length header on
        the string is 4, but the fourth byte is not a null terminator.
        """,
        error=corruption_at(16),
    ),
    TestCase(
        'dbpointer/invalid-utf8/accept',
        doc(
            elem(
                'foo',
                Tag.DBPointer,
                string(b'abc\xffdef'),  # String with invalid UTF-8
                b'\x52\x59\xb5\x6a\xfa\x5b\xd8\x41\xd6\x58\x5d\x99',  # OID
            )
        ),
        """
        DBPointer document, but the collection string contains invalid UTF-8
        """,
    ),
    TestCase(
        'dbpointer/invalid-utf8/reject',
        doc(
            elem(
                'foo',
                Tag.DBPointer,
                string(b'abc\xffdef'),  # String with invalid UTF-8
                b'\x52\x59\xb5\x6a\xfa\x5b\xd8\x41\xd6\x58\x5d\x99',  # OID
            )
        ),
        """
        DBPointer document, but the collection string contains invalid UTF-8
        """,
        flags=BSON_VALIDATE_UTF8,
        error=ErrorInfo(BSON_VALIDATE_UTF8, 'Text element is not valid UTF-8', 4),
    ),
    TestCase(
        'dbpointer/invalid-utf8/accept-if-absent',
        doc(
            elem(
                'foo',
                Tag.DBPointer,
                string(b'abcdef'),  # Valid string
                b'\x52\x59\xb5\x6a\xfa\x5b\xd8\x41\xd6\x58\x5d\x99',  # OID
            )
        ),
        """
        DBPointer document, and we validate UTF-8. Accepts because there is no
        invalid UTF-8 here.
        """,
        flags=BSON_VALIDATE_UTF8,
    ),
    TestCase(
        'subdoc/simple',
        doc(elem('doc', Tag.Document, doc(utf8elem('foo', 'bar')))),
        """
        A simple document: { doc: { foo: "bar" } }
        """,
    ),
    TestCase(
        'subdoc/invalid-shared-null',
        doc(elem('doc', Tag.Document, doc()[:-1])),
        """
        A truncated subdocument element, with its null terminator accidentally
        overlapping the parent document's null.
        """,
        error=corruption_at(9),
    ),
    TestCase(
        'subdoc/overlapping-utf8-null',
        doc(elem('doc', Tag.Document, doc(utf8elem('bar', 'baz\0')[:-1]))),
        """
        Encodes the document:

            { "foo": { "bar": "baz" } }

        but the foo.bar UTF-8 string is truncated improperly and reuses the null
        terminator for "foo"
        """,
        error=corruption_at(18),
    ),
    TestCase(
        'subdoc/invalid-element',
        doc(elem('doc', Tag.Document, doc(elem('dbl', Tag.Double, b'abcd')))),
        'A subdocument that contains an invalid element',
        error=corruption_at(18),
    ),
    TestCase(
        'subdoc/header-too-large',
        doc(
            elem(
                'foo',
                Tag.Document,
                b'\xf7\xff\xff\xff\0',  # Bad document
            ),
        ),
        """
        Data {"foo": {}}, but the subdoc header is too large.
        """,
        error=corruption_at(9),
    ),
    TestCase(
        'subdoc/header-too-small',
        doc(
            elem(
                'test',
                Tag.Document,
                b'\x04\0\0\0',  # Only four bytes. All docs must be at least 5
            ),
        ),
        """
        Nested document with a header value of 4, which is always too small.
        """,
        error=corruption_at(4),
    ),
    TestCase(
        'subdoc/impossible-size',
        doc(
            elem(
                'foo',
                Tag.Document,
                b'\xff\xff\xff\xff\0',  # Bad document
            ),
        ),
        """
        Data {"foo": {}}, but the subdoc header is UINT32_MAX/INT32_MIN, which
        becomes is an invalid document header.
        """,
        error=corruption_at(9),
    ),
    TestCase(
        'null/simple',
        doc(elem('null', Tag.Null)),
        """
        A simple document: { "null": null }
        """,
    ),
    TestCase(
        'undefined/simple',
        doc(elem('undefined', Tag.Undefined)),
        """
        A simple document: { "undefined": undefined }
        """,
    ),
    TestCase(
        'binary/simple',
        doc(elem('binary', Tag.Binary, binary(0x80, b'12345'))),
        """
        Simple binary data { "binary": Binary(0x80, b'12345') }
        """,
    ),
    TestCase(
        'binary/bad-length-zero-subtype-2',
        doc(
            elem(
                'binary',
                Tag.Binary,
                i32le(0),  # Invalid: Zero length
                b'\x02',  # subtype two
                i32le(4),  # Length of 4
                b'1234',  # payload
            ),
        ),
        """
        Binary data that has an invalid length header. It is subtype 2,
        which means it contains an additional length header.
        """,
        error=corruption_at(12),
    ),
    TestCase(
        'binary/bad-inner-length-on-subtype-2',
        doc(
            elem(
                'binary',
                Tag.Binary,
                i32le(8),  # Valid length
                b'\x02',  # subtype two
                i32le(2),  # Invalid length of (should be 4)
                b'1234',  # payload
            ),
        ),
        """
        Binary data that has an valid outer length header, but the inner length
        header for subtype 2 has an incorrect value.
        """,
        error=corruption_at(17),
    ),
    TestCase(
        'binary/bad-length-too-small',
        doc(
            elem(
                'binary',
                Tag.Binary,
                i32le(2),  # Length prefix (too small)
                b'\x80',  # subtype
                b'1234',  # payload
            ),
        ),
        """
        Data { "binary": Binary(0x80, b'1234') }, but the length header on
        the Binary object is too small.

        This won't cause the binary to decode wrong, but it will cause the iterator
        to jump into the middle of the binary data which will not decode as a
        proper BSON element.
        """,
        error=corruption_at(22),
    ),
    TestCase(
        'binary/bad-length-too-big',
        doc(
            elem(
                'binary',
                Tag.Binary,
                b'\xf3\xff\xff\xff',  # Length prefix (too big)
                b'\x80',  # subtype
                b'1234',  # data
            ),
        ),
        """
        Data { "binary": Binary(0x80, b'1234') }, but the length header on
        the Binary object is too large.
        """,
        error=corruption_at(12),
    ),
    TestCase(
        'binary/old-invalid/1',
        doc(
            elem(
                'binary',
                Tag.Binary,
                binary(
                    2,
                    i32le(5),  # Bad length prefix: Should be 4
                    b'abcd',
                ),
            ),
        ),
        """
        This is an old-style binary type 0x2. It has an inner length header of 5,
        but it should be 4.
        """,
        error=corruption_at(17),
    ),
    TestCase(
        'binary/old-invalid/2',
        doc(
            elem(
                'bin',
                Tag.Binary,
                binary(
                    2,
                    b'abc',  # Bad: Subtype 2 requires at least four bytes
                ),
            )
        ),
        """
        This is an old-style binary type 0x2. The data segment is too small to
        be valid.
        """,
        error=corruption_at(9),
    ),
    TestCase(
        'minkey/simple',
        doc(elem('min', Tag.MinKey)),
        'A simple document with a MinKey element',
    ),
    TestCase(
        'maxkey/simple',
        doc(elem('max', Tag.MaxKey)),
        'A simple document with a MaxKey element',
    ),
    TestCase(
        'int32/simple',
        doc(elem('int32', Tag.Int32, i32le(42))),
        'A simple document with a valid single int32 element',
    ),
    TestCase(
        'int32/truncated',
        doc(elem('int32-truncated', Tag.Int32, i32le(42)[:-1])),
        'Truncated 32-bit integer',
        error=corruption_at(21),
    ),
    TestCase('timestamp/simple', doc(elem('timestamp', Tag.Timestamp, i64le(1729))), """A simple timestamp element"""),
    TestCase(
        'timestamp/truncated',
        doc(elem('timestamp', Tag.Timestamp, i64le(1729)[:-1])),
        """A truncated timestamp element""",
        error=corruption_at(15),
    ),
    TestCase(
        'int64/simple',
        doc(elem('int64', Tag.Int64, i64le(1729))),
        'A simple document with a valid single int64 element',
    ),
    TestCase(
        'int64/truncated',
        doc(elem('int64-truncated', Tag.Int64, i64le(1729)[:-1])),
        'Truncated 64-bit integer',
        error=corruption_at(21),
    ),
    TestCase(
        'double/simple',
        doc(elem('double', Tag.Double, f64le(3.14))),
        'Simple float64 element',
    ),
    TestCase(
        'double/truncated',
        doc(elem('double-truncated', Tag.Double, f64le(3.13)[:-1])),
        'Truncated 64-bit float',
        error=corruption_at(22),
    ),
    TestCase(
        'boolean/simple-false',
        doc(elem('bool', Tag.Boolean, b'\x00')),
        """A simple boolean 'false'""",
    ),
    TestCase(
        'boolean/simple-true',
        doc(elem('bool', Tag.Boolean, b'\x01')),
        """A simple boolean 'true'""",
    ),
    TestCase(
        'boolean/invalid',
        doc(elem('bool', Tag.Boolean, b'\xc3')),
        """
        An invalid boolean octet. Must be '0' or '1', but is 0xc3.
        """,
        error=corruption_at(10),
    ),
    TestCase(
        'datetime/simple',
        doc(elem('utc', Tag.Datetime, b'\x0b\x98\x8c\x2b\x33\x01\x00\x00')),
        'Simple datetime element',
    ),
    TestCase(
        'datetime/truncated',
        doc(elem('utc', Tag.Datetime, b'\x0b\x98\x8c\x2b\x33\x01\x00')),
        'Truncated datetime element',
        error=corruption_at(9),
    ),
    # DBRef
    TestCase(
        'dbref/missing-id',
        doc(utf8elem('$ref', 'foo')),
        """This dbref document is missing an $id element""",
        flags=BSON_VALIDATE_DOLLAR_KEYS,
        error=ErrorInfo(BSON_VALIDATE_DOLLAR_KEYS, MSG_EXPECTED_ID_FOLLOWING_REF, 18),
    ),
    TestCase(
        'dbref/non-id',
        doc(utf8elem('$ref', 'foo'), utf8elem('bar', 'baz')),
        """
        The 'bar' element should be an '$id' element.
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
        error=ErrorInfo(BSON_VALIDATE_DOLLAR_KEYS, MSG_EXPECTED_ID_FOLLOWING_REF, 18),
    ),
    TestCase(
        'dbref/not-first-elements',
        doc(utf8elem('foo', 'bar'), utf8elem('$ref', 'a'), utf8elem('$id', 'b')),
        """
        This would be a valid DBRef, but the "$ref" key must come first.
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
        error=ErrorInfo(BSON_VALIDATE_DOLLAR_KEYS, disallowed_key('$', '$ref'), 17),
    ),
    TestCase(
        'dbref/ref-without-id-with-db',
        doc(utf8elem('$ref', 'foo'), utf8elem('$db', 'bar')),
        """
        There should be an $id element, but we skip straight to $db
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
        error=ErrorInfo(BSON_VALIDATE_DOLLAR_KEYS, MSG_EXPECTED_ID_FOLLOWING_REF, 18),
    ),
    TestCase(
        'dbref/non-string-ref',
        doc(elem('$ref', Tag.Int32, i32le(42))),
        """
        The $ref element must be a string, but is an integer.
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
        error=ErrorInfo(BSON_VALIDATE_DOLLAR_KEYS, '$ref element must be a UTF-8 element', 4),
    ),
    TestCase(
        'dbref/non-string-db',
        doc(
            utf8elem('$ref', 'foo'),
            utf8elem('$id', 'bar'),
            elem('$db', Tag.Int32, i32le(42)),
        ),
        """
        The $db element should be a string, but is an integer.
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
        error=ErrorInfo(BSON_VALIDATE_DOLLAR_KEYS, '$db element in DBRef must be a UTF-8 element', 31),
    ),
    TestCase(
        'dbref/invalid-extras-between',
        doc(
            utf8elem('$ref', 'foo'),
            utf8elem('$id', 'bar'),
            utf8elem('extra', 'field'),
            utf8elem('$db', 'baz'),
        ),
        """
        Almost a valid DBRef, but there is an extra field before $db. We reject $db
        as an invalid key.
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
        error=ErrorInfo(BSON_VALIDATE_DOLLAR_KEYS, disallowed_key('$', '$db'), 48),
    ),
    TestCase(
        'dbref/invalid-double-ref',
        doc(
            utf8elem('$ref', 'foo'),
            utf8elem('$ref', 'bar'),
            utf8elem('$id', 'baz'),
        ),
        """
        Invalid DBRef contains a second $ref element.
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
        error=ErrorInfo(BSON_VALIDATE_DOLLAR_KEYS, MSG_EXPECTED_ID_FOLLOWING_REF, 18),
    ),
    TestCase(
        'dbref/invalid-missing-ref',
        doc(utf8elem('$id', 'foo')),
        """
        DBRef document requires a $ref key to be first.
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
        error=ErrorInfo(BSON_VALIDATE_DOLLAR_KEYS, disallowed_key('$', '$id'), 4),
    ),
    TestCase(
        'dbref/valid/simple',
        doc(utf8elem('$ref', 'foo'), utf8elem('$id', 'bar')),
        """
        This is a simple valid DBRef element.
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
    ),
    TestCase(
        'dbref/valid/simple-with-db',
        doc(utf8elem('$ref', 'foo'), utf8elem('$id', 'bar'), utf8elem('$db', 'baz')),
        """
        A simple DBRef of the form:

            { $ref: "foo", $id: "bar", $db: "baz" }
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
    ),
    TestCase(
        'dbref/valid/nested-id-doc',
        doc(
            utf8elem('$ref', 'foo'),
            elem(
                '$id',
                Tag.Document,
                doc(
                    utf8elem('$ref', 'foo2'),
                    utf8elem('$id', 'bar2'),
                    utf8elem('$db', 'baz2'),
                ),
            ),
            utf8elem('$db', 'baz'),
        ),
        """
        This is a valid DBRef of the form:

            { $ref: foo, $id: { $ref: "foo2", $id: "bar2", $db: "baz2" }, $db: "baz" }
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
    ),
    TestCase(
        'dbref/valid/trailing-content',
        doc(
            utf8elem('$ref', 'foo'),
            utf8elem('$id', 'bar'),
            utf8elem('$db', 'baz'),
            utf8elem('extra', 'field'),
        ),
        """
        A valid DBRef of the form:

            {
                $ref: "foo",
                $id: "bar",
                $db: "baz",
                extra: "field",
            }
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
    ),
    TestCase(
        'dbref/valid/trailing-content-no-db',
        doc(
            utf8elem('$ref', 'foo'),
            utf8elem('$id', 'bar'),
            utf8elem('extra', 'field'),
        ),
        """
        A valid DBRef of the form:

            {
                $ref: "foo",
                $id: "bar",
                extra: "field",
            }
        """,
        flags=BSON_VALIDATE_DOLLAR_KEYS,
    ),
]

if __name__ == '__main__':
    # We don't take an arguments, but error if any are given
    parser = argparse.ArgumentParser(description=__doc__)
    parser.parse_args()
    # Start with the header
    print(HEADER)
    # Print each test case
    for c in CASES:
        print()
        for part in generate(c):
            print(part, end='')

    # Print the registration function
    print(f'\n{GENERATED_NOTE}')
    print('void test_install_generated_bson_validation(TestSuite* suite) {')
    for c in CASES:
        print(f'  TestSuite_Add(suite, "/bson/validate/" {json.dumps(c.name)}, {c.fn_name});')
    print('}')



from config_generator.components.cse.openssl import SaslCyrusOpenSSLCompile
from config_generator.components.sanitizers.asan import TAG
from config_generator.etc.compile import generate_compile_tasks
from config_generator.etc.sanitizers.test import generate_test_tasks

# pylint: disable=line-too-long
# fmt: off
COMPILE_MATRIX = [
    ('rhel8-latest',     'clang', None, ['cyrus']),
]

# CSFLE requires 4.2+. QE requires 7.0+ and are skipped on "server" tasks.
TEST_MATRIX = [
    # rhel8-latest provides 4.2 through latest.
    ('rhel8-latest', 'clang', None, 'cyrus', ['auth'], ['server', 'replica'], ['4.2', '4.4', '5.0', '6.0', '7.0', '8.0', 'latest']),
]
# fmt: on
# pylint: enable=line-too-long


MORE_TAGS = ['cse', 'asan']


def tasks():
    res = []

    SSL = 'openssl'
    SASL_TO_FUNC = {
        'cyrus': SaslCyrusOpenSSLCompile,
    }

    res += generate_compile_tasks(SSL, TAG, SASL_TO_FUNC, COMPILE_MATRIX, MORE_TAGS)

    res += generate_test_tasks(SSL, TAG, TEST_MATRIX, MORE_TAGS)

    res += generate_test_tasks(
        SSL, TAG, TEST_MATRIX, MORE_TAGS, MORE_TEST_TAGS=['with-mongocrypt'], MORE_VARS={'SKIP_CRYPT_SHARED_LIB': 'on'}
    )

    return res



from shrub.v3.evg_command import EvgCommandType
from shrub.v3.evg_task import EvgTask

from config_generator.etc.function import Function
from config_generator.etc.utils import bash_exec


class KmsDivergenceCheck(Function):
    name = 'kms-divergence-check'
    commands = [
        bash_exec(
            command_type=EvgCommandType.TEST,
            working_dir='mongoc',
            script='.evergreen/scripts/kms-divergence-check.sh',
        ),
    ]


def functions():
    return KmsDivergenceCheck.defn()


def tasks():
    return [
        EvgTask(
            name=KmsDivergenceCheck.name,
            commands=[
                KmsDivergenceCheck.call(),
            ],
        )
    ]



from pydantic import ConfigDict
from shrub.v3.evg_build_variant import BuildVariant
from shrub.v3.evg_command import (
    BuiltInCommand,
    EvgCommandType,
    KeyValueParam,
    ec2_assume_role,
    expansions_update,
    s3_put,
)
from shrub.v3.evg_task import EvgTask, EvgTaskRef

from config_generator.etc.distros import find_small_distro
from config_generator.etc.function import Function, merge_defns
from config_generator.etc.utils import bash_exec

TAG = 'sbom'


class CustomCommand(BuiltInCommand):
    command: str
    model_config = ConfigDict(arbitrary_types_allowed=True)


class SBOM(Function):
    name = 'sbom'
    commands = [
        # Authenticate with Kondukto.
        *[
            ec2_assume_role(
                command_type=EvgCommandType.SETUP,
                role_arn='${kondukto_role_arn}',
            ),
            bash_exec(
                command_type=EvgCommandType.SETUP,
                include_expansions_in_env=[
                    'AWS_ACCESS_KEY_ID',
                    'AWS_SECRET_ACCESS_KEY',
                    'AWS_SESSION_TOKEN',
                ],
                script="""\
                set -o errexit
                set -o pipefail
                kondukto_token="$(aws secretsmanager get-secret-value --secret-id "kondukto-token" --region "us-east-1" --query 'SecretString' --output text)"
                printf "KONDUKTO_TOKEN: %s\\n" "$kondukto_token" >|expansions.kondukto.yml
                """,
            ),
            expansions_update(
                command_type=EvgCommandType.SETUP,
                file='expansions.kondukto.yml',
            ),
        ],
        # Authenticate with Amazon ECR.
        *[
            # Avoid inadvertently using a pre-existing and potentially conflicting Podman config.
            # Note: podman understands and uses DOCKER_CONFIG despite the name.
            expansions_update(updates=[KeyValueParam(key='DOCKER_CONFIG', value='${workdir}/.docker')]),
            ec2_assume_role(role_arn='arn:aws:iam::901841024863:role/ecr-role-evergreen-ro'),
            bash_exec(
                command_type=EvgCommandType.SETUP,
                include_expansions_in_env=[
                    'AWS_ACCESS_KEY_ID',
                    'AWS_SECRET_ACCESS_KEY',
                    'AWS_SESSION_TOKEN',
                    'DOCKER_CONFIG',
                ],
                script='aws ecr get-login-password --region us-east-1 | podman login --username AWS --password-stdin 901841024863.dkr.ecr.us-east-1.amazonaws.com',
            ),
        ],
        bash_exec(
            command_type=EvgCommandType.TEST,
            working_dir='mongoc',
            include_expansions_in_env=[
                'branch_name',
                'DOCKER_CONFIG',
                'KONDUKTO_TOKEN',
            ],
            script='.evergreen/scripts/sbom.sh',
        ),
        s3_put(
            command_type=EvgCommandType.TEST,
            aws_key='${aws_key}',
            aws_secret='${aws_secret}',
            bucket='mciuploads',
            content_type='application/json',
            display_name='Augmented SBOM',
            local_file='mongoc/augmented-sbom.json',
            permissions='public-read',
            remote_file='${project}/${build_variant}/${revision}/${version_id}/${build_id}/sbom/augmented-sbom.json',
        ),
    ]


def functions():
    return merge_defns(
        SBOM.defn(),
    )


def tasks():
    distro_name = 'rhel80'
    distro = find_small_distro(distro_name)

    yield EvgTask(
        name='sbom',
        tags=[TAG, distro_name],
        run_on=distro.name,
        commands=[
            SBOM.call(),
        ],
    )


def variants():
    return [
        BuildVariant(
            name=TAG,
            display_name='SBOM',
            tasks=[EvgTaskRef(name=f'.{TAG}')],
        ),
    ]
